"""
Fixtures for metrics
"""

# Remove this when changing over to object model
# as this is repeated within the check_template

metrics_common_template = {
    "check": {
        "state": {
            "running": "false",
            "killed": "false",
            "configured": "true",
            "disabled": "false",
            "target_ip": "23.253.6.64",
            "last_run": {
                "@now": "1422323039.361",
                "#text": "1422323039.357"
            },
            "runtime": "0.958",
            "availability": "available",
            "state": "good",
            "status": "ok",
            "metrics": [
                {
                    "@type": "inprogress"
                },
                {
                    "@type": "current",
                    "@timestamp": "1422323039.357"
                }
            ]
        }
    }
}


metrics = {
    "selfcheck": {
        "metric": [
            {"@name": "version",
             "@type": "s",
             "#text": "ckdev-stage.8e17ed475b8a80103d11ec29e5e122fe256f8bf7.1416497041"},
            {"@name": "check_cnt",
             "@type": "i",
             "#text": "5"},
            {"@name": "transient_cnt",
             "@type": "i",
             "#text": "0"},
            {"@name": "uptime",
             "@type": "l",
             "#text": "165"},
            {"@name": "metrics_collected",
             "@type": "L",
             "#text": "321"},
            {"@name": "feed_bytes",
             "@type": "l",
             "#text": "23817"},
            {"@name": "default_queue_threads",
             "@type": "i",
             "#text": "10"},
            {"@name": "checks_run",
             "@type": "L",
             "#text": "50"}
        ]
    },
    "ping_icmp": {
        "metric": [
            {"@name": "available",
             "@type": "n",
             "#text": "0.000000000000e+00"},
            {"@name": "count",
             "@type": "i",
             "#text": "2"},
            {"@name": "maximum",
             "@type": "n"},
            {"@name": "minimum",
             "@type": "n"},
            {"@name": "average",
             "@type": "n"}
        ]

    },
    "tcp": {
        "metric": [
            {"@name": "banner",
             "@type": "s",
             "#text": "test"},
            {"@name": "test_banner_match",
             "@type": "s",
             "#text": "test"},
            {"@name": "body_match",
             "@type": "s",
             "#text": "test_body_match"},
            {"@name": "duration",
             "@type": "i",
             "#text": "30"},
            {"@name": "tt_body",
             "@type": "i",
             "#text": "1"},
            {"@name": "tt_connect",
             "@type": "i",
             "#text": "2"},
            {"@name": "tt_firstbyte",
             "@type": "i",
             "#text": "3"}
        ]
    },
    "http": {
        "metric": [
            {
                "@name": "cert_end",
                "@type": "I",
                "#text": "1471910399"
            },
            {
                "@name": "truncated",
                "@type": "I",
                "#text": "0"
            },
            {
                "@name": "cert_subject",
                "@type": "s",
                "#text": ("\/C=US\/ST=Texas\/L=San Antonio\/O=Rackspace US,"
                          " Inc.\/OU=Marketing\/CN=www.rackspace.com")
            },
            {
                "@name": "cert_start",
                "@type": "I",
                "#text": "1415059200"
            },
            {
                "@name": "cert_issuer",
                "@type": "s",
                "#text": ("\/C=US\/O=Symantec Corporation\/OU=Symantec"
                          " Trust Network\/CN=Symantec Class 3 Secure Server CA - G4")
            },
            {
                "@name": "code",
                "@type": "s",
                "#text": "200"
            },
            {
                "@name": "tt_connect",
                "@type": "I",
                "#text": "72"
            },
            {
                "@name": "cert_end_in",
                "@type": "i",
                "#text": "49587360"
            },
            {
                "@name": "tt_firstbyte",
                "@type": "I",
                "#text": "957"
            },
            {
                "@name": "bytes",
                "@type": "i",
                "#text": "44779"
            },
            {
                "@name": "cert_subject_alternative_names",
                "@type": "s",
                "#text": ("wwwp.wip.rackspace.com, ord.wwwp.wip.rackspace.com,"
                          " iad.wwwp.wip.rackspace.com, admin.rackspace.com,"
                          " iad.wip.rackspace.com, ord.wip.rackspace.com, www.rackspace.com")
            },
            {
                "@name": "cert_error",
                "@type": "s",
                "#text": ("No certificate present., host header "
                          "does not match CN or SANs in certificate")
            },
            {
                "@name": "duration",
                "@type": "I",
                "#text": "957"
            }
        ]
    }
}



"""
Tests for :mod:`nova_api` and :mod:`nova_objects` for flavors.
"""

from twisted.trial.unittest import SynchronousTestCase

from mimic.test.helpers import json_request, request
from mimic.rest.nova_api import NovaApi
from mimic.test.fixtures import APIMockHelper


class NovaAPIFlavorsTests(SynchronousTestCase):

    """
    Tests for Flavors using the Nova Api plugin.
    """

    def setUp(self):
        """
        Create a :obj:`MimicCore` with :obj:`NovaApi` as the only plugin.
        """
        nova_api = NovaApi(['DFW'])
        self.helper = APIMockHelper(self, [nova_api])
        self.root = self.helper.root
        self.uri = self.helper.uri

    def get_server_flavor(self, postfix):
        """
        Get flavors, assert response code is 200 and return response body.
        """
        (response, content) = self.successResultOf(json_request(
            self, self.root, "GET", self.uri + postfix))
        self.assertEqual(200, response.code)
        return content

    def test_get_flavor(self):
        """
        Test to verify :func:`get_flavor` on ``GET /v2.0/<tenant_id>/flavors/<flavor_id>``
        """
        get_server_flavor = self.get_server_flavor('/flavors/2')
        self.assertEqual(get_server_flavor['flavor']['id'], '2')

    def test_get_server_flavor_negative(self):
        """
        Test to verify :func:`get_flavor` when invalid flavor from the
        :obj: `mimic_presets` is provided.
        """
        get_server_flavor = request(self, self.root, "GET", self.uri +
                                    '/flavors/negative-test-1')
        get_server_flavor_response = self.successResultOf(get_server_flavor)
        self.assertEqual(get_server_flavor_response.code, 404)

    def test_get_flavor_list(self):
        """
        Test to verify :func:`get_flavor_list` on ``GET /v2.0/<tenant_id>/flavors``
        """
        get_flavor_list_response_body = self.get_server_flavor('/flavors')
        flavor_list = get_flavor_list_response_body['flavors']
        self.assertTrue(len(flavor_list) > 1)
        for each_flavor in flavor_list:
            self.assertEqual(sorted(each_flavor.keys()), sorted(['id', 'name', 'links']))

    def test_get_flavor_list_with_OnMetal_negative(self):
        """
        Test to verify :func:`get_flavor_list` on ``GET /v2.0/<tenant_id>/flavors``
        does not have OnMetal flavors in regions other than IAD
        """
        get_flavor_list_response_body = self.get_server_flavor('/flavors/detail')
        flavor_list = get_flavor_list_response_body['flavors']
        self.assertTrue(len(flavor_list) == 35)
        for flavor in flavor_list:
            self.assertNotEqual('onmetal_flavor',
                                flavor['OS-FLV-WITH-EXT-SPECS:extra_specs']['policy_class'])

    def test_get_flavor_list_with_OnMetal(self):
        """
        Test to verify :func:`get_flavor_list` on ``GET /v2.0/<tenant_id>/flavors``
        """
        nova_api = NovaApi(['IAD'])
        self.helper = APIMockHelper(self, [nova_api])
        self.root = self.helper.root
        self.uri = self.helper.uri
        get_flavor_list_response_body = self.get_server_flavor('/flavors')
        flavor_list = get_flavor_list_response_body['flavors']
        self.assertTrue(len(flavor_list) == 38)
        for flavor in flavor_list:
            if 'onmetal' in flavor['id']:
                self.assertTrue('onmetal' in flavor['id'])
                self.assertEqual(
                    sorted(flavor.keys()),
                    sorted(['id', 'name', 'links']))

        get_flavor_list_response_body = self.get_server_flavor('/flavors/detail')
        flavor_list = get_flavor_list_response_body['flavors']
        self.assertTrue(len(flavor_list) == 38)
        for flavor in flavor_list:
            if 'onmetal' in flavor['id']:
                self.assertEqual('onmetal_flavor',
                                 flavor['OS-FLV-WITH-EXT-SPECS:extra_specs']['policy_class'])
                self.assertEqual(
                    sorted(flavor.keys()),
                    sorted(['id', 'name', 'links', 'ram', 'OS-FLV-WITH-EXT-SPECS:extra_specs',
                            'vcpus', 'swap', 'rxtx_factor', 'OS-FLV-EXT-DATA:ephemeral', 'disk']))

    def test_get_flavor_list_with_details(self):
        """
        Test to verify :func:`test_get_flavor_list_with_details` on
        ``GET /v2.0/<tenant_id>/flavors/details``
        """
        get_flavor_list_response_body = self.get_server_flavor('/flavors/detail')
        flavor_list = get_flavor_list_response_body['flavors']
        self.assertTrue(len(flavor_list) > 1)
        for each_flavor in flavor_list:
            self.assertEqual(
                sorted(each_flavor.keys()),
                sorted(['id', 'name', 'links', 'ram', 'OS-FLV-WITH-EXT-SPECS:extra_specs',
                        'vcpus', 'swap', 'rxtx_factor', 'OS-FLV-EXT-DATA:ephemeral', 'disk']))

    def test_get_flavor_list_with_details_is_consistent(self):
        """
        Test to verify ``GET /v2.0/<tenant_id>/flavors/details`` returns
        consistent data on mutiple list calls
        """
        get_flavor_list_response_body = self.get_server_flavor('/flavors/detail')
        get_flavor_list2_response_body = self.get_server_flavor('/flavors/detail')
        self.assertEqual(get_flavor_list_response_body, get_flavor_list2_response_body)

    def test_get_flavor_list_after_list_flavors(self):
        """
        Test to verify :func:`get_flavor_list` on ``GET /v2.0/<tenant_id>/flavors``
        http://docs.rackspace.com/servers/api/v2/cs-devguide/content/List_Flavors-d1e4188.html
        """
        flavor_list = self.get_server_flavor('/flavors')['flavors']
        for each_flavor in flavor_list:
            flavor_id = each_flavor['id']
            name = each_flavor['name']
            get_server_flavor_response_body = self.get_server_flavor('/flavors/' + flavor_id)
            self.assertEqual(flavor_id, get_server_flavor_response_body['flavor']['id'])
            self.assertEqual(name, get_server_flavor_response_body['flavor']['name'])

    def test_get_OnMetal_flavor(self):
        """
        Test to verify :func:`get_flavor` on ``GET /v2.0/<tenant_id>/flavors/<flavor_id>``
        """
        get_server_flavor = self.get_server_flavor('/flavors/onmetal-compute1')
        self.assertEqual(get_server_flavor['flavor']['id'], 'onmetal-compute1')
        self.assertEqual(sorted(get_server_flavor['flavor']['OS-FLV-WITH-EXT-SPECS:extra_specs'].keys()),
                         sorted(['quota_resources', 'class', 'policy_class']))




from twisted.trial.unittest import SynchronousTestCase
from twisted.internet.task import Clock

from mimic.core import MimicCore
from mimic.resource import MimicRoot
from mimic.test.helpers import json_request


class ValkyrieAPITests(SynchronousTestCase):

    """
    Tests for the Valkyrie API
    """

    def setUp(self):
        """
        Initialize core and root
        """
        self.core = MimicCore(Clock(), [])
        self.root = MimicRoot(self.core).app.resource()
        self.url = "/valkyrie/v2.0"

    def test_post_auth_token_to_login_endpoint(self):
        """
        Obtain an auth token
        """
        data = {"something": "anything"}
        (response, content) = self.successResultOf(json_request(self, self.root,
                                                                "POST",
                                                                self.url + "/login", data))
        self.assertEqual(200, response.code)

    def test_post_auth_token_to_login_user_endpoint(self):
        """
        Obtain an auth token
        """
        data = {"something": "anything"}
        (response, content) = self.successResultOf(json_request(self, self.root,
                                                                "POST",
                                                                self.url + "/login_user", data))
        self.assertEqual(200, response.code)

    def test_get_devices_effective_permissions(self):
        """
        Obtain list of device permissions for contact 12 on account 123456
        """
        (response, content) = self.successResultOf(
            json_request(self, self.root, "GET",
                         self.url +
                         "/account/123456/permissions/contacts/devices/by_contact/12/effective"))
        self.assertEqual(200, response.code)
        self.assertTrue(content["contact_permissions"])
        self.assertEqual(len(content["contact_permissions"]), 4)

    def test_get_empty_accounts_effective_permissions(self):
        """
        Obtain list of account permissions for contact 12 on account 123456
        """
        (response, content) = self.successResultOf(
            json_request(self, self.root, "GET",
                         self.url +
                         "/account/123456/permissions/contacts/accounts/by_contact/12/effective"))
        self.assertEqual(200, response.code)
        self.assertFalse(content["contact_permissions"])

    def test_get_accounts_effective_permissions(self):
        """
        Obtain list of account permissions for contact 12 on account 123456
        """
        (response, content) = self.successResultOf(
            json_request(self, self.root, "GET",
                         self.url +
                         "/account/123456/permissions/contacts/accounts/by_contact/34/effective"))
        self.assertEqual(200, response.code)
        self.assertTrue(content["contact_permissions"])
        self.assertEqual(len(content["contact_permissions"]), 1)
        self.assertEqual(content["contact_permissions"][0]["permission_type"], 15)
        self.assertEqual(content["contact_permissions"][0]["item_type_name"], "accounts")

    def test_get_empty_devices_effective_permissions(self):
        """
        Obtain list of devices permissions for contact 34 on account 123456
        """
        (response, content) = self.successResultOf(
            json_request(self, self.root, "GET",
                         self.url +
                         "/account/123456/permissions/contacts/devices/by_contact/34/effective"))
        self.assertEqual(200, response.code)
        self.assertFalse(content["contact_permissions"])

    def test_get_devices_permissions_item_id(self):
        """
        Obtain list of device permissions for contact 78 on account 654321
        """
        (response, content) = self.successResultOf(
            json_request(self, self.root, "GET",
                         self.url +
                         "/account/654321/permissions/contacts/devices/by_contact/78/effective"))
        self.assertEqual(200, response.code)
        self.assertTrue(content["contact_permissions"])
        self.assertEqual(len(content["contact_permissions"]), 1)
        permission = content["contact_permissions"][0]
        self.assertEqual(permission["permission_type"], 14)
        self.assertEqual(permission["item_id"], 262144)
        self.assertEqual(permission["item_type_name"], "devices")

    def test_get_any_permissions(self):
        """
        Obtain list of all permissions for contact 90 on account 654321
        """
        (response, content) = self.successResultOf(
            json_request(self, self.root, "GET",
                         self.url +
                         "/account/654321/permissions/contacts/any/by_contact/90/effective"))
        self.assertEqual(200, response.code)
        self.assertTrue(content["contact_permissions"])
        self.assertEqual(len(content["contact_permissions"]), 2)
        device_permission = content["contact_permissions"][0]
        self.assertEqual(device_permission["permission_type"], 12)
        self.assertEqual(device_permission["item_id"], 1048576)
        self.assertEqual(device_permission["item_type_name"], "devices")
        account_permission = content["contact_permissions"][1]
        self.assertEqual(account_permission["permission_type"], 15)
        self.assertEqual(account_permission["item_id"], 654321)
        self.assertEqual(account_permission["item_type_name"], "accounts")



"""
Define fixtures to provide common functionality for Mimic testing
"""
from mimic.test.helpers import json_request
from mimic.core import MimicCore
from mimic.resource import MimicRoot
from twisted.internet.task import Clock


class TenantAuthentication(object):
    """
    Provides some functionality to help log into mimic identity with a
    particular username and password
    """
    def __init__(self, test_case, root, username, password):
        """
        Authenticate a particular user against the mimic root.

        :param root: The :class:`twisted.web.resource.IResource` at the root
            of the mimic API resource tree.

        :param username: the username to authenticate as
        :param password: the password with which to use to authenticate
        """
        _, self.service_catalog_json = test_case.successResultOf(json_request(
            test_case, root, "POST", "/identity/v2.0/tokens",
            {
                "auth": {
                    "passwordCredentials": {
                        "username": username,
                        "password": password,
                    },
                }
            }
        ))

    def get_service_endpoint(self, service_name, region=''):
        """
        Return the publicURL for the given service and region. Note that if there are multiple
        endpoints for a given region, the first will be returned, and if no region is specified,
        the first endpoint will be returned.
        :param unicode service_name: The name of the service for which to get an endpoint as
            listed in the service catalog
        :param unicode region: The service catalog region of the desired endpoint
        """
        for service in self.service_catalog_json['access']['serviceCatalog']:
            if service['name'] == service_name:
                for item in service['endpoints']:
                    if (item['region'] == region) or (region == ''):
                        return item['publicURL']
        raise KeyError("No such service {}".format(service_name))


class APIMockHelper(object):
    """
    Provides common functionality for mimic tests
    """

    def __init__(self, test_case, apis):
        """
        Initialize a mimic core and the specified :obj:`mimic.imimic.IAPIMock`s
        :param apis: A list of :obj:`mimic.imimic.IAPIMock` objects to be initialized
        """
        self.test_case = test_case
        self.clock = Clock()
        self.core = MimicCore(self.clock, apis)
        self.root = MimicRoot(self.core).app.resource()

        # Pass in arbitrary username and password
        self.auth = TenantAuthentication(test_case, self.root,
                                         "test1", "test1password")

        # map some attributes and methods
        self.service_catalog_json = self.auth.service_catalog_json
        self.get_service_endpoint = self.auth.get_service_endpoint

        tenant_id = self.auth.service_catalog_json["access"]["token"]["tenant"]["id"]
        service_name = apis[0].catalog_entries(tenant_id)[0].name
        self.uri = self.get_service_endpoint(service_name)



import treq
import json

from twisted.trial.unittest import SynchronousTestCase
from twisted.internet.task import Clock

from mimic.test.fixtures import APIMockHelper
from mimic.test.helpers import request, json_request
from mimic.rest.glance_api import GlanceApi
from mimic.core import MimicCore
from mimic.resource import MimicRoot
from mimic.canned_responses.json.glance.glance_images_json import image_schema
from mimic.model.glance_objects import random_image_list


class GlanceAPITests(SynchronousTestCase):
    """
    Tests for the Glance plugin api
    """

    def get_responsebody(self, r):
        """
        util json response body
        """
        return self.successResultOf(treq.json_content(r))

    def setUp(self):
        """
        Setup for glance tests
        """
        helper = APIMockHelper(self, [GlanceApi()])
        self.root = helper.root
        self.uri = helper.uri

    def test_list_images(self):
        """
        List the images returned from glance
        """
        req = request(self, self.root, "GET", self.uri + '/images', '')
        resp = self.successResultOf(req)
        self.assertEquals(resp.code, 200)
        data = self.get_responsebody(resp)
        self.assertEquals(True, 'images' in json.dumps(data))


class GlanceAdminAPITests(SynchronousTestCase):
    """
    Tests for the Glance Admin API
    """

    def setUp(self):
        """
        Initialize core and root
        """
        self.core = MimicCore(Clock(), [])
        self.root = MimicRoot(self.core).app.resource()
        self.uri = "/glance/v2/images"
        self.create_request = {"name": "OnMetal - MIMIC", "distro": "linux"}

    def create_image(self, request_json=None):
        """
        Create image and validate response code.
        Return newly created image.
        """
        request_json = request_json or self.create_request
        (response, content) = self.successResultOf(json_request(
            self, self.root, "POST", self.uri,
            body=request_json))
        self.assertEqual(response.code, 201)
        return content

    def list_images(self):
        """
        List images and return response
        """
        (response, content) = self.successResultOf(json_request(
            self, self.root, "GET", self.uri))
        self.assertEqual(200, response.code)
        for each in content['images']:
            self.assertEqual(each["status"], "active")
        return content

    def test_list_image_schema(self):
        """
        Get the image schema returned from glance admin API
        """
        uri = "/glance/v2/schemas/image"
        (response, content) = self.successResultOf(json_request(
            self, self.root, "GET", uri))
        self.assertEqual(200, response.code)
        self.assertEqual(sorted(image_schema.keys()),
                         sorted(content))

    def test_list_images_for_admin(self):
        """
        List the images returned from the glance admin api
        """
        content = self.list_images()
        self.assertEqual(len(random_image_list), len(content['images']))
        actual_image_names = [image['name'] for image in content['images']]
        expected_image_names = [each['name'] for each in random_image_list]
        self.assertEqual(sorted(actual_image_names), sorted(expected_image_names))

    def test_list_images_for_admin_consistently(self):
        """
        List the images returned from the glance admin api
        """
        content1 = self.list_images()
        content2 = self.list_images()
        self.assertEqual(content2, content1)

    def test_create_image(self):
        """
        Create Image and validate response
        """
        new_image = self.create_image()
        self.assertEqual(new_image['name'], self.create_request['name'])

    def test_create_image_fails_with_400(self):
        """
        Create Image and validate response
        """
        request_jsons = [{}, {"name": None}, {"hello": "world"}]
        for each in request_jsons:
            (response, content) = self.successResultOf(json_request(
                self, self.root, "POST", self.uri,
                body=json.dumps(each)))
            self.assertEqual(response.code, 400)

    def test_get_image(self):
        """
        Create then GET Image and validate response
        """
        new_image = self.create_image()

        (response, content) = self.successResultOf(json_request(
            self, self.root, "GET", self.uri + '/' + new_image['id']))
        self.assertEqual(200, response.code)
        self.assertEqual(new_image, content)

    def test_get_non_existant_image(self):
        """
        Return 404 when trying to GET a non existant image.
        """
        response = self.successResultOf(request(
            self, self.root, "GET", self.uri + '/' + '1111'))
        self.assertEqual(404, response.code)

    def test_delete_non_existant_image(self):
        """
        Return 404 when trying to DELETE a non existant image.
        """
        response = self.successResultOf(request(
            self, self.root, "DELETE", self.uri + '/' + '1111'))
        self.assertEqual(404, response.code)

    def test_delete_image(self):
        """
        Create and then delete Image and validate response
        """
        new_image = self.create_image()

        response = self.successResultOf(request(
            self, self.root, "DELETE", self.uri + '/' + new_image['id']))
        self.assertEqual(204, response.code)

        response = self.successResultOf(request(
            self, self.root, "GET", self.uri + '/' + new_image['id']))
        self.assertEqual(404, response.code)

    def test_get_then_delete_image(self):
        """
        Create and then delete Image and validate response
        """
        images = self.list_images()['images']

        for each in images[:2]:
            response = self.successResultOf(request(
                self, self.root, "DELETE", self.uri + '/' + each['id']))
            self.assertEqual(204, response.code)

            response = self.successResultOf(request(
                self, self.root, "GET", self.uri + '/' + each['id']))
            self.assertEqual(404, response.code)

        images_after_delete = self.list_images()['images']
        self.assertEqual(len(images),
                         len(images_after_delete) + 2)



import json

from twisted.trial.unittest import SynchronousTestCase
from twisted.internet.task import Clock

from mimic.core import MimicCore
from mimic.resource import MimicRoot
from mimic.test.helpers import json_request, request


class CustomerAPITests(SynchronousTestCase):

    """
    Tests for the Customer API
    """

    def setUp(self):
        """
        Initialize core and root.

        `default_contacts` is the default contact set for any tenant,
        when the contacts are not set using ``POST /contacts``.
        """
        self.core = MimicCore(Clock(), [])
        self.root = MimicRoot(self.core).app.resource()
        self.default_contacts = [{"email": "example@example.com", "role": "TECHNICAL"},
                                 {"email": "example2@example.com", "role": "TECHNICAL"}]

    def get_contacts(self, root, tenant):
        """
        Get contacts and verify that the responce code was 200.
        Returns the contacts list
        """
        (response, content) = self.successResultOf(json_request(
            self, root, "GET",
            "/v1/customer_accounts/CLOUD/{0}/contacts".format(tenant)))
        self.assertEqual(200, response.code)
        return content

    def validate_contact_list(self, response, expected_contacts):
        """
        Validate that contacts in the `response` is same as the
        `expected_contacts`
        """
        self.assertEqual(len(response["contact"]), len(expected_contacts))
        contact_emails = [each_contact["emailAddresses"]["emailAddress"][0]["address"]
                          for each_contact in response["contact"]]
        self.assertTrue(expected_contacts[0]["email"] in contact_emails)
        self.assertTrue(expected_contacts[1]["email"] in contact_emails)
        contact_roles = [each_contact["roles"]["role"][0]
                         for each_contact in response["contact"]]
        self.assertTrue(expected_contacts[0]["role"] in contact_roles)
        self.assertTrue(expected_contacts[1]["role"] in contact_roles)

    def test_get_default_contacts_for_a_tenant_successfully(self):
        """
        The ``GET /contacts`` call returns a list of default contacts
        for a tenant with the response code 200.
        """
        response = self.get_contacts(self.root, "111111")
        self.validate_contact_list(response, self.default_contacts)

    def test_get_default_contacts_for_multiple_tenants_successfully(self):
        """
        The ``GET /contacts`` call returns a list of default contacts
        for multiple tenants with the response code 200.
        """
        for each_tenant in ['111111', '22222', '33333', '444444']:
            response = self.get_contacts(self.root, each_tenant)
            self.validate_contact_list(response, self.default_contacts)

    def test_set_contacts_for_a_tenant_successfully(self):
        """
        The ``POST /contacts`` call adds contacts to a tenant, which is then
        listed by the ``GET /contacts`` calls
        """
        expected_contacts = [{"email": "test@email.com", "role": "PRIMARY"},
                             {"email": "new@email.com", "role": "TECHNICAL"}]
        response = self.successResultOf(request(
            self, self.root, "POST",
            "/v1/customer_accounts/CLOUD/555555/contacts",
            json.dumps(expected_contacts)))
        self.assertEqual(200, response.code)
        response = self.get_contacts(self.root, "555555")
        self.validate_contact_list(response, expected_contacts)

    def test_change_contacts_from_default_contacts(self):
        """
        The ``GET /contacts`` call returns a list of default contacts then
        change the contacts using ``POST /contacts``
        """
        response = self.get_contacts(self.root, "111111")
        self.validate_contact_list(response, self.default_contacts)
        expected_contacts = [{"email": "test1@email.com", "role": "PRIMARY"},
                             {"email": "new1@email.com", "role": "TECHNICAL"}]
        response = self.successResultOf(request(
            self, self.root, "POST",
            "/v1/customer_accounts/CLOUD/555555/contacts",
            json.dumps(expected_contacts)))
        self.assertEqual(200, response.code)
        response = self.get_contacts(self.root, "555555")
        self.validate_contact_list(response, expected_contacts)

    def test_set_contacts_to_be_null(self):
        """
        The ``POST /contacts`` call adds empty contacts to a tenant, which
        is then listed by the ``GET /contacts`` calls
        """
        expected_contacts = []
        response = self.successResultOf(request(
            self, self.root, "POST",
            "/v1/customer_accounts/CLOUD/77777/contacts",
            json.dumps(expected_contacts)))
        self.assertEqual(200, response.code)
        response = self.get_contacts(self.root, "77777")
        self.assertEqual(len(response["contact"]), 0)



"""
Tests for mimic identity (:mod:`mimic.model.identity` and
:mod:`mimic.rest.auth_api`)
"""
import json

from twisted.trial.unittest import SynchronousTestCase
from twisted.internet.task import Clock

from mimic.canned_responses.auth import (
    get_token, HARD_CODED_TOKEN, HARD_CODED_USER_ID,
    HARD_CODED_USER_NAME, HARD_CODED_ROLES,
    get_endpoints
)
from mimic.canned_responses.mimic_presets import get_presets
from mimic.catalog import Entry, Endpoint
from mimic.core import MimicCore
from mimic.resource import MimicRoot
from mimic.test.behavior_tests import (
    behavior_tests_helper_class,
    register_behavior
)
from mimic.test.dummy import ExampleAPI
from mimic.test.helpers import json_request, request, request_with_content


def core_and_root(api_list):
    """
    Given a list of APIs to load, return core and root.
    """
    core = MimicCore(Clock(), api_list)
    root = MimicRoot(core).app.resource()
    return core, root


class ExampleCatalogEndpoint(object):

    def __init__(self, tenant, num, endpoint_id):
        self._tenant = tenant
        self._num = num
        self.endpoint_id = endpoint_id

    @property
    def region(self):
        return "EXAMPLE_{num}".format(num=self._num)

    @property
    def tenant_id(self):
        return "{tenant}_{num}".format(tenant=self._tenant,
                                       num=self._num)

    def url_with_prefix(self, prefix):
        return "http://ok_{num}".format(num=self._num)


class ExampleCatalogEntry(object):

    """
    Example of a thing that a plugin produces at some phase of its lifecycle;
    maybe you have to pass it a tenant ID to get one of these.  (Services which
    don't want to show up in the catalog won't produce these.)
    """

    def __init__(self, tenant_id, name, endpoint_count=2, idgen=lambda: 1):
        # some services transform their tenant ID
        self.name = name
        self.type = "compute"
        self.path_prefix = "/v2/"
        self.endpoints = [ExampleCatalogEndpoint(tenant_id, n + 1, idgen())
                          for n in range(endpoint_count)]


def example_endpoints(counter):
    """
    Create some example catalog entries from a given tenant ID, like the plugin
    loader would.
    """
    def endpoints(tenant_id):
        yield ExampleCatalogEntry(tenant_id, "something", idgen=counter)
        yield ExampleCatalogEntry(tenant_id, "something_else", idgen=counter)
    return endpoints


class CatalogGenerationTests(SynchronousTestCase):

    """
    Tests for generating a service catalog in various formats from a common
    data source.
    """

    # Service catalogs are pretty large, so set the testing option to a value
    # where we can see as much as possible of the difference in the case of a
    # failure.
    maxDiff = None

    def test_tokens_response(self):
        """
        :func:`get_token` returns JSON-serializable data in the format
        presented by a ``POST /v2.0/tokens`` API request; i.e. the normal
        user-facing service catalog generation.
        """
        tenant_id = 'abcdefg'
        self.assertEqual(
            get_token(
                tenant_id=tenant_id, timestamp=lambda dt: "<<<timestamp>>>",
                entry_generator=example_endpoints(lambda: 1),
                prefix_for_endpoint=lambda e: 'prefix'
            ),
            {
                "access": {
                    "token": {
                        "id": HARD_CODED_TOKEN,
                        "expires": "<<<timestamp>>>",
                        "tenant": {
                            "id": tenant_id,
                            "name": tenant_id,  # TODO: parameterize later
                        },
                        "RAX-AUTH:authenticatedBy": [
                            "PASSWORD",
                        ]
                    },
                    "serviceCatalog": [
                        {
                            "name": "something",
                            "type": "compute",
                            "endpoints": [
                                {
                                    "region": "EXAMPLE_1",
                                    "tenantId": "abcdefg_1",
                                    "publicURL": "http://ok_1"
                                },
                                {
                                    "region": "EXAMPLE_2",
                                    "tenantId": "abcdefg_2",
                                    "publicURL": "http://ok_2"
                                }
                            ]
                        },
                        {
                            "name": "something_else",
                            "type": "compute",
                            "endpoints": [
                                {
                                    "region": "EXAMPLE_1",
                                    "tenantId": "abcdefg_1",
                                    "publicURL": "http://ok_1"
                                },
                                {
                                    "region": "EXAMPLE_2",
                                    "tenantId": "abcdefg_2",
                                    "publicURL": "http://ok_2"
                                }
                            ]
                        }
                    ],
                    "user": {
                        "id": HARD_CODED_USER_ID,
                        "name": HARD_CODED_USER_NAME,
                        "roles": HARD_CODED_ROLES,
                    }
                }
            }
        )

    def test_endpoints_response(self):
        """
        :func:`get_endpoints` returns JSON-serializable data in the format
        presented by a ``GET /v2.0/tokens/<token>/endpoints``; i.e. the
        administrative list of tokens.
        """
        tenant_id = 'abcdefg'
        from itertools import count
        accum = count(1)

        def counter():
            return next(accum)
        # Possible TODO for cloudServersOpenStack:

        # "versionInfo": "http://localhost:8902/v2",
        # "versionList": "http://localhost:8902/",
        # "versionId": "2",

        self.assertEqual(
            get_endpoints(
                tenant_id=tenant_id,
                entry_generator=example_endpoints(counter),
                prefix_for_endpoint=lambda e: 'prefix'
            ),
            {
                "endpoints": [
                    {
                        "region": "EXAMPLE_1",
                        "tenantId": "abcdefg_1",
                        "publicURL": "http://ok_1",
                        "name": "something",
                        "type": "compute",
                        "id": 1,
                    },
                    {
                        "region": "EXAMPLE_2",
                        "tenantId": "abcdefg_2",
                        "publicURL": "http://ok_2",
                        "name": "something",
                        "type": "compute",
                        "id": 2,
                    },
                    {
                        "region": "EXAMPLE_1",
                        "tenantId": "abcdefg_1",
                        "publicURL": "http://ok_1",
                        "name": "something_else",
                        "type": "compute",
                        "id": 3,
                    },
                    {
                        "region": "EXAMPLE_2",
                        "tenantId": "abcdefg_2",
                        "publicURL": "http://ok_2",
                        "name": "something_else",
                        "type": "compute",
                        "id": 4
                    }
                ]
            },
        )

    def test_unversioned_entry(self):
        """
        An L{Endpoint} created without a 'prefix' returns a URI without a
        version.
        """
        self.assertEqual(
            get_endpoints(
                tenant_id="1234",
                entry_generator=lambda t_id: [Entry(
                    tenant_id=t_id, type="compute",
                    name="compute_name", endpoints=[
                        Endpoint(tenant_id=t_id,
                                 region="None",
                                 endpoint_id="eid")
                    ]
                )],
                prefix_for_endpoint=lambda ep: "http://prefix/"
            ),
            {
                "endpoints": [
                    {
                        "id": "eid",
                        "name": "compute_name",
                        "type": "compute",
                        "region": "None",
                        "tenantId": "1234",
                        "publicURL": "http://prefix/1234"
                    }
                ]
            }
        )


def authenticate_with_username_password(test_case, root,
                                        uri='/identity/v2.0/tokens',
                                        username=None, password=None,
                                        tenant_name=None, tenant_id=None,
                                        request_func=json_request):
    """
    Returns a tuple of the response code and json body after authentication
    with username and password.
    """
    creds = {
        "auth": {
            "passwordCredentials": {
                "username": username or "demoauthor",
                "password": password or "theUsersPassword"
            }

        }
    }
    if tenant_id is not None:
        creds["auth"]["tenantId"] = tenant_id
    if tenant_name is not None:
        creds["auth"]["tenantName"] = tenant_name
    return test_case.successResultOf(request_func(test_case, root, "POST",
                                                  uri, json.dumps(creds)))


def authenticate_with_api_key(test_case, root, uri='/identity/v2.0/tokens',
                              username=None, api_key=None,
                              tenant_name=None, tenant_id=None):
    """
    Returns a tuple of the response code and json body after authentication
    using the username and api_key.
    """
    creds = {
        "auth": {
            "RAX-KSKEY:apiKeyCredentials": {
                "username": username or "demoauthor",
                "apiKey": api_key or "jhgjhghg-nhghghgh-12222"
            }

        }
    }
    if tenant_id is not None:
        creds["auth"]["tenantId"] = tenant_id
    if tenant_name is not None:
        creds["auth"]["tenantName"] = tenant_name
    return test_case.successResultOf(json_request(test_case, root, "POST",
                                                  uri, creds))


def authenticate_with_token(test_case, root, uri='/identity/v2.0/tokens',
                            token_id=None, tenant_id=None):
    """
    Returns a tuple of the response code and json body after authentication
    using token and tenant ids.
    """
    creds = {
        "auth": {
            "tenantId": tenant_id or "12345",
            "token": {
                "id": token_id or "iuyiuyiuy-uyiuyiuy-1987878"
            }
        }
    }
    return test_case.successResultOf(json_request(test_case, root, "POST",
                                                  uri, creds))


def impersonate_user(test_case, root,
                     uri="http://mybase/identity/v2.0/RAX-AUTH/impersonation-tokens",
                     username=None, impersonator_token=None):
    """
    Returns a tuple of the response code and json body after authentication
    using token and tenant ids.
    """
    headers = {
        'X-Auth-Token': [str(impersonator_token)]} if impersonator_token else None
    return test_case.successResultOf(json_request(
        test_case, root, "POST", uri,
        {"RAX-AUTH:impersonation": {"expire-in-seconds": 30,
                                    "user": {"username": username or "test1"}}},
        headers=headers
    ))


class GetAuthTokenAPITests(SynchronousTestCase):

    """
    Tests for ``/identity/v2.0/tokens``, provided by
    :obj:`mimic.rest.auth_api.AuthApi.get_token_and_service_catalog`
    """

    def test_response_has_auth_token(self):
        """
        The JSON response has a access.token.id key corresponding to its
        MimicCore session, and therefore access.token.tenant.id should match
        that session's tenant_id.
        """
        core, root = core_and_root([])
        (response, json_body) = authenticate_with_username_password(self, root)
        self.assertEqual(200, response.code)
        token = json_body['access']['token']['id']
        tenant_id = json_body['access']['token']['tenant']['id']
        session = core.sessions.session_for_token(token)
        self.assertEqual(token, session.token)
        self.assertEqual(tenant_id, session.tenant_id)

    def test_response_has_user_admin_identity_role(self):
        """
        The JSON response for authenticate has the role `identity:user-admin`.
        """
        core, root = core_and_root([])
        (response, json_body) = authenticate_with_username_password(self, root)
        self.assertEqual(200, response.code)
        self.assertEqual(
            json_body['access']['user']['roles'], HARD_CODED_ROLES)

    def test_response_has_same_roles_despite_number_of_auths(self):
        """
        The JSON response for authenticate has only one `identity:user-admin`
        role, no matter how many times the user authenticates.
        """
        core, root = core_and_root([])
        (response, json_body) = authenticate_with_username_password(self, root)
        self.assertEqual(200, response.code)
        self.assertEqual(
            json_body['access']['user']['roles'], HARD_CODED_ROLES)
        (response1, json_body1) = authenticate_with_username_password(
            self, root)
        self.assertEqual(200, response1.code)
        self.assertEqual(
            json_body1['access']['user']['roles'], HARD_CODED_ROLES)
        (response2, json_body2) = authenticate_with_username_password(
            self, root)
        self.assertEqual(200, response2.code)
        self.assertEqual(
            json_body2['access']['user']['roles'], HARD_CODED_ROLES)

    def test_authentication_request_with_no_body_causes_http_bad_request(self):
        """
        The response for empty body request is bad_request.
        """
        core, root = core_and_root([])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "POST", "/identity/v2.0/tokens", ""))

        self.assertEqual(400, response.code)

    def test_authentication_request_with_invalid_body_causes_http_bad_request(self):
        """
        The response for not JSON body request is bad_request.
        """
        core, root = core_and_root([])

        response = self.successResultOf(request(
            self, root, "POST", "/identity/v2.0/tokens", "{ bad request: }"))

        self.assertEqual(400, response.code)

    def test_auth_accepts_tenant_name(self):
        """
        If "tenantName" is passed, the tenant specified is used instead of a
        generated tenant ID.
        """
        core, root = core_and_root([])

        (response, json_body) = authenticate_with_username_password(
            self,
            root,
            tenant_name="turtlepower")

        self.assertEqual(200, response.code)
        self.assertEqual("turtlepower",
                         json_body['access']['token']['tenant']['id'])
        token = json_body['access']['token']['id']
        session = core.sessions.session_for_token(token)
        self.assertEqual(token, session.token)
        self.assertEqual("turtlepower", session.tenant_id)

    def test_auth_accepts_tenant_id(self):
        """
        If "tenantId" is passed, the tenant specified is used instead of a
        generated tenant ID.
        """
        core, root = core_and_root([])
        (response, json_body) = authenticate_with_username_password(
            self,
            root,
            tenant_id="turtlepower")
        self.assertEqual(200, response.code)
        self.assertEqual("turtlepower",
                         json_body['access']['token']['tenant']['id'])
        token = json_body['access']['token']['id']
        session = core.sessions.session_for_token(token)
        self.assertEqual(token, session.token)
        self.assertEqual("turtlepower", session.tenant_id)

    def test_response_service_catalog_has_base_uri(self):
        """
        The JSON response's service catalog whose endpoints all begin with
        the same base URI as the request.
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_username_password(
            self,
            root, uri='http://mybase/identity/v2.0/tokens')
        self.assertEqual(200, response.code)
        services = json_body['access']['serviceCatalog']
        self.assertEqual(1, len(services))

        urls = [
            endpoint['publicURL'] for endpoint in services[0]['endpoints']
        ]
        self.assertEqual(1, len(urls))
        self.assertTrue(urls[0].startswith('http://mybase/'),
                        '{0} does not start with "http://mybase"'
                        .format(urls[0]))


class GetEndpointsForTokenTests(SynchronousTestCase):

    """
    Tests for ``/identity/v2.0/tokens/<token>/endpoints``, provided by
    `:obj:`mimic.rest.auth_api.AuthApi.get_endpoints_for_token`
    """

    def test_session_created_for_token(self):
        """
        A session is created for the token provided
        """
        core, root = core_and_root([])

        token = '1234567890'

        request(
            self, root, "GET",
            "/identity/v2.0/tokens/{0}/endpoints".format(token)
        )

        session = core.sessions.session_for_token(token)
        self.assertEqual(token, session.token)

    def test_response_service_catalog_has_base_uri(self):
        """
        The JSON response's service catalog whose endpoints all begin with
        the same base URI as the request.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/1234567890/endpoints"
        ))

        self.assertEqual(200, response.code)
        urls = [endpoint['publicURL'] for endpoint in json_body['endpoints']]
        self.assertEqual(1, len(urls))

        self.assertTrue(
            urls[0].startswith('http://mybase/'),
            '{0} does not start with "http://mybase"'.format(urls[0]))

    def test_api_service_endpoints_are_not_duplicated(self):
        """
        The service catalog should not duplicate endpoints for an entry/endpoints
        """
        regions_and_versions_list = [
            ("ORD", "v1"), ("DFW", "v1"), ("DFW", "v2"), ("IAD", "v3")]
        core, root = core_and_root([ExampleAPI(
            regions_and_versions=regions_and_versions_list)])

        (response, json_body) = authenticate_with_username_password(self, root)
        self.assertEqual(response.code, 200)
        service_catalog = json_body["access"]["serviceCatalog"]
        self.assertEqual(len(service_catalog), 1)
        endpoints_list = service_catalog[0]["endpoints"]
        self.assertEqual(len(endpoints_list), 4)

    def test_get_token_and_catalog_for_password_credentials(self):
        """
        The response returned should include the password credentials that were supplied
        during authentication
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_username_password(self, root,
                                                                    tenant_id='12345')
        self.assertEqual(response.code, 200)
        tenant_id = json_body["access"]["token"]["tenant"]["id"]
        self.assertEqual(tenant_id, "12345")
        tenant_name = json_body["access"]["token"]["tenant"]["name"]
        self.assertEqual(tenant_name, tenant_id)
        user_name = json_body["access"]["user"]["name"]
        self.assertEqual(user_name, "demoauthor")

    def test_get_token_and_catalog_for_api_credentials(self):
        """
        The response returned should include the credentials that were supplied
        during authentication
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_api_key(self, root,
                                                          tenant_name='12345')
        self.assertEqual(response.code, 200)
        tenant_id = json_body["access"]["token"]["tenant"]["id"]
        self.assertEqual(tenant_id, "12345")
        tenant_name = json_body["access"]["token"]["tenant"]["name"]
        self.assertEqual(tenant_name, tenant_id)
        user_name = json_body["access"]["user"]["name"]
        self.assertEqual(user_name, "demoauthor")

    def test_get_token_and_catalog_for_token_credentials(self):
        """
        The response returned should include the credentials that were supplied
        during authentication
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_token(
            self, root, tenant_id='12345')
        self.assertEqual(response.code, 200)
        tenant_id = json_body["access"]["token"]["tenant"]["id"]
        self.assertEqual(tenant_id, "12345")
        tenant_name = json_body["access"]["token"]["tenant"]["name"]
        self.assertEqual(tenant_name, tenant_id)
        user_name = json_body["access"]["user"]["name"]
        self.assertTrue(user_name)

    def test_token_and_catalog_for_password_credentials_wrong_tenant(self):
        """
        Tenant ID is validated when provided in username/password auth.

        If authed once as one tenant ID, and a second time with a different
        tenant ID, then the second auth will return with a 401 Unauthorized.
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_username_password(
            self, root, tenant_id="12345")
        self.assertEqual(response.code, 200)
        username = json_body["access"]["user"]["id"]

        (response, fail_body) = authenticate_with_username_password(
            self, root, tenant_id="23456")
        self.assertEqual(response.code, 401)
        self.assertEqual(fail_body, {
            "unauthorized": {
                "code": 401,
                "message": ("Tenant with Name/Id: '23456' is not valid for "
                            "User 'demoauthor' (id: '{0}')".format(username))
            }
        })

    def test_rax_kskey_apikeycredentials(self):
        """
        Test apiKeyCredentials
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "/identity/v2.0/users/1/OS-KSADM/credentials/RAX-KSKEY:apiKeyCredentials"
        ))
        self.assertEqual(response.code, 404)
        self.assertEqual(
            json_body['itemNotFound']['message'], 'User 1 not found')
        creds = {
            "auth": {
                "passwordCredentials": {
                    "username": "HedKandi",
                    "password": "Ministry Of Sound UK"
                },
                "tenantId": "77777"
            }
        }
        (response, json_body) = self.successResultOf(json_request(
            self, root, "POST", "/identity/v2.0/tokens", creds))
        self.assertEqual(response.code, 200)
        user_id = json_body['access']['user']['id']
        username = json_body['access']['user']['name']
        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "/identity/v2.0/users/" + user_id +
            "/OS-KSADM/credentials/RAX-KSKEY:apiKeyCredentials"
        ))
        self.assertEqual(response.code, 200)
        self.assertEqual(json_body['RAX-KSKEY:apiKeyCredentials']['username'],
                         username)
        self.assertTrue(
            len(json_body['RAX-KSKEY:apiKeyCredentials']['apiKey']) == 32)

    def test_token_and_catalog_for_api_credentials_wrong_tenant(self):
        """
        Tenant ID is validated when provided in api-key auth.

        If authed once as one tenant ID, and a second time with a different
        tenant ID, then the second auth will return with a 401 Unauthorized.
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_api_key(
            self, root, tenant_id="12345")
        self.assertEqual(response.code, 200)
        username = json_body["access"]["user"]["id"]

        (response, fail_body) = authenticate_with_api_key(
            self, root, tenant_id="23456")
        self.assertEqual(response.code, 401)
        self.assertEqual(fail_body, {
            "unauthorized": {
                "code": 401,
                "message": ("Tenant with Name/Id: '23456' is not valid for "
                            "User 'demoauthor' (id: '{0}')".format(username))
            }
        })

    def test_token_and_catalog_for_token_credentials_wrong_tenant(self):
        """
        Tenant ID is validated when provided in token auth.

        If authed once as one tenant ID, and a second time with a different
        tenant ID, then the second auth will return with a 401 Unauthorized.
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_token(
            self, root, tenant_id="12345")
        self.assertEqual(response.code, 200)

        (response, fail_body) = authenticate_with_token(
            self, root, tenant_id="23456")
        self.assertEqual(response.code, 401)
        self.assertEqual(fail_body, {
            "unauthorized": {
                "code": 401,
                "message": ("Token doesn't belong to Tenant with Id/Name: "
                            "'23456'")
            }
        })

    def test_get_token_and_catalog_for_invalid_json_request_body(self):
        """
        :func: `get_token_and_service_catalog` returns response code 400, when
        an invalid json request body is used to authenticate.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "POST", "/identity/v2.0/tokens",
            {
                "auth": {
                    "token": {
                        "id": "iuyiuyiuy-uyiuyiuy-1987878"
                    }
                }
            }
        ))
        self.assertEqual(response.code, 400)
        self.assertEqual(json_body["message"], "Invalid JSON request body")

    def test_response_for_get_username(self):
        """
        Test to verify :func: `get_username`.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v1.1/mosso/123456"
        ))
        self.assertEqual(301, response.code)
        self.assertTrue(json_body['user']['id'])

    def test_response_for_impersonation(self):
        """
        Test to verify :func: `get_impersonation_token`.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = impersonate_user(self, root)
        self.assertEqual(200, response.code)
        self.assertTrue(json_body['access']['token']['id'])

    def test_impersonation_without_expires_in_seconds_attribute(self):
        """
        Test to verify :func: `get_impersonation_token` when the `expire-in-seconds`
        is not provided.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "POST", "http://mybase/identity/v2.0/RAX-AUTH/impersonation-tokens",
            {"RAX-AUTH:impersonation": {"user": {"username": "user-test"}}}))
        self.assertEqual(200, response.code)
        self.assertTrue(json_body['access']['token']['id'])

    def test_impersonation_request_with_no_body_causes_http_bad_request(self):
        """
        The response for empty body request is bad_request.
        """
        core, root = core_and_root([])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "POST", "http://mybase/identity/v2.0/RAX-AUTH/impersonation-tokens", ""))

        self.assertEqual(400, response.code)

    def test_impersonation_request_with_invalid_body_causes_http_bad_request(self):
        """
        The response for not JSON body request is bad_request.
        """
        core, root = core_and_root([])

        response = self.successResultOf(request(
            self, root, "POST", "http://mybase/identity/v2.0/RAX-AUTH/impersonation-tokens",
                                "{ bad request: }"))

        self.assertEqual(400, response.code)

    def test_response_for_validate_token(self):
        """
        Test to verify :func: `validate_token`.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/123456a?belongsTo=111111"
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body['access']['token']['id'], '123456a')
        self.assertTrue(json_body['access']['user']['id'])
        self.assertTrue(len(json_body['access']['user']['roles']) > 0)
        self.assertTrue(json_body['access'].get('serviceCatalog') is None)

    def test_response_for_validate_token_when_tenant_not_provided(self):
        """
        Test to verify :func: `validate_token` when tenant_id is not
        provided using the argument `belongsTo`
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/123456a"
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body['access']['token']['id'], '123456a')
        self.assertTrue(json_body['access']['token']['tenant']['id'])

    def test_response_for_validate_token_then_authenticate(self):
        """
        Test to verify :func: `validate_token` and then authenticate
        """
        core, root = core_and_root([ExampleAPI()])

        (response1, json_body1) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/123456a?belongsTo=111111"
        ))
        self.assertEqual(200, response1.code)
        (response, json_body) = authenticate_with_token(self, root,
                                                        tenant_id="111111",
                                                        token_id="123456a")
        self.assertEqual(response.code, 200)
        self.assertEqual(json_body["access"]["token"]["id"],
                         json_body1["access"]["token"]["id"])
        self.assertEqual(json_body["access"]["token"]["tenant"]["id"],
                         json_body1["access"]["token"]["tenant"]["id"])
        self.assertEqual(json_body["access"]["user"]["name"],
                         json_body1["access"]["user"]["name"])

    def test_response_for_validate_impersonated_token(self):
        """
        Test to verify :func: `validate_token` and then authenticate
        """
        core, root = core_and_root([ExampleAPI()])

        # Authenticate the impersonator (admin user)
        (response0, json_body0) = authenticate_with_token(
            self, root,
            tenant_id="111111",
            token_id="123456a")
        self.assertEqual(200, response0.code)
        impersonator_token = json_body0["access"]["token"]["id"]

        # Authenticate using the username so we know the tenant_id
        (response1, json_body1) = authenticate_with_username_password(
            self, root,
            username="test1",
            tenant_id="12345")
        self.assertEqual(200, response1.code)

        # Impersonate user test1
        (response2, json_body2) = impersonate_user(
            self, root,
            username="test1",
            impersonator_token=impersonator_token)
        self.assertEqual(200, response2.code)
        impersonated_token = json_body2["access"]["token"]["id"]

        # validate the impersonated_token
        (response3, json_body3) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/{0}?belongsTo=12345".format(
                impersonated_token)
        ))
        self.assertEqual(200, response3.code)
        self.assertTrue(json_body3["access"]["RAX-AUTH:impersonator"])

    def test_response_for_validate_impersonated_token_multiple_users(self):
        """
        Test to verify :func: `validate_token` and then authenticate
        """
        core, root = core_and_root([ExampleAPI()])

        # Authenticate the impersonator (admin user 1)
        (response0, json_body0) = authenticate_with_token(
            self, root,
            tenant_id="111111",
            token_id="123456a")
        self.assertEqual(200, response0.code)
        impersonator_token1 = json_body0["access"]["token"]["id"]

        # Authenticate the impersonator (admin user 2)
        (response1, json_body1) = authenticate_with_token(
            self, root,
            tenant_id="222222",
            token_id="123456b")

        self.assertEqual(200, response1.code)
        impersonator_token2 = json_body1["access"]["token"]["id"]

        # Authenticate the impersonatee using the username so we know the
        # tenant_id to make the validate token id call with 'belongsTo'
        (response2, json_body2) = authenticate_with_username_password(
            self, root,
            username="test1",
            tenant_id="12345")
        self.assertEqual(200, response2.code)

        # Impersonate user test1 using admin user1's token
        (response3, json_body3) = impersonate_user(
            self, root,
            username="test1",
            impersonator_token=impersonator_token1)
        self.assertEqual(200, response3.code)
        impersonated_token1 = json_body3["access"]["token"]["id"]

        # Impersonate user test1 using admin user2's token
        (response4, json_body4) = impersonate_user(
            self, root,
            username="test1",
            impersonator_token=impersonator_token2)
        self.assertEqual(200, response4.code)
        impersonated_token2 = json_body4["access"]["token"]["id"]

        # validate the impersonated_token1
        (response5, json_body5) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/{0}?belongsTo=12345".format(
                impersonated_token1)
        ))
        self.assertEqual(200, response5.code)
        self.assertTrue(json_body5["access"]["RAX-AUTH:impersonator"])
        self.assertEqual(json_body5["access"]["RAX-AUTH:impersonator"]["name"],
                         json_body0["access"]["user"]["name"])

        # validate the impersonated_token2
        (response6, json_body6) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/{0}?belongsTo=12345".format(
                impersonated_token2)
        ))
        self.assertEqual(200, response6.code)
        self.assertTrue(json_body6["access"]["RAX-AUTH:impersonator"])
        self.assertEqual(json_body6["access"]["RAX-AUTH:impersonator"]["name"],
                         json_body1["access"]["user"]["name"])

    def test_response_for_validate_token_with_maas_admin_role(self):
        """
        Test to verify :func: `validate_token` when the token_id provided
        is of an maas admin user specified in `mimic_presets`.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/this_is_an_impersonator_token"
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body["access"]["RAX-AUTH:impersonator"]["roles"][0]["name"],
                         "monitoring:service-admin")

    def test_response_for_validate_token_with_racker_role(self):
        """
        Test to verify :func: `validate_token` when the token_id provided
        is of a racker specified in `mimic_presets`.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/this_is_a_racker_token"
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body["access"]["RAX-AUTH:impersonator"]["roles"][0]["name"],
                         "Racker")

    def test_response_for_validate_token_when_invalid(self):
        """
        Test to verify :func: `validate_token` when the token_id provided
        is invalid, as specified in `mimic_presets`.
        """
        core, root = core_and_root([ExampleAPI()])
        token = get_presets["identity"]["token_fail_to_auth"][0]

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/{0}".format(token)
        ))
        self.assertEqual(401, response.code)

    def test_response_for_validate_token_with_observer_role(self):
        """
        Test to verify :func: `validate_token` when the tenant_id provided
        is of an observer role, as specified in `mimic_presets`.
        """
        core, root = core_and_root([ExampleAPI()])
        token = get_presets["identity"]["observer_role"][0]

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/any_token?belongsTo={0}".format(token)
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body["access"]["user"]["roles"][0]["name"],
                         "observer")
        self.assertEqual(json_body["access"]["user"]["roles"][0]["description"],
                         "Global Observer Role.")

    def test_response_for_validate_token_with_creator_role(self):
        """
        Test to verify :func: `validate_token` when the tenant_id provided
        is of an creator role, as specified in `mimic_presets`.
        """
        core, root = core_and_root([ExampleAPI()])
        token = get_presets["identity"]["creator_role"][0]

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/any_token?belongsTo={0}".format(token)
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body["access"]["user"]["roles"][0]["name"],
                         "creator")
        self.assertEqual(json_body["access"]["user"]["roles"][0]["description"],
                         "Global Creator Role.")

    def test_response_for_validate_token_with_admin_and_observer_role(self):
        """
        Test to verify :func: `validate_token` when the tenant_id provided
        is of an admin role, as specified in `mimic_presets`.
        """
        core, root = core_and_root([ExampleAPI()])
        token = get_presets["identity"]["admin_role"][0]

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/tokens/any_token?belongsTo={0}".format(token)
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body["access"]["user"]["roles"][0]["name"],
                         "admin")
        self.assertEqual(json_body["access"]["user"]["roles"][0]["description"],
                         "Global Admin Role.")
        self.assertEqual(json_body["access"]["user"]["roles"][1]["name"],
                         "observer")
        self.assertEqual(json_body["access"]["user"]["roles"][1]["description"],
                         "Global Observer Role.")

    def test_response_for_list_users(self):
        """
        Test to verify :func: `get_users_details`.
        """
        core, root = core_and_root([ExampleAPI()])

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/users?name=random_user"
        ))
        self.assertEqual(200, response.code)
        self.assertTrue(json_body['user']['RAX-AUTH:domainId'])

    def test_response_for_list_users_after_authentication(self):
        """
        Test to verify :func: `get_users_details`.
        """
        core, root = core_and_root([ExampleAPI()])
        (response, json_body) = authenticate_with_token(
            self, root, tenant_id="12345")
        self.assertEqual(response.code, 200)
        username = json_body["access"]["user"]["name"]
        user_id = json_body["access"]["user"]["id"]

        (response, json_body) = self.successResultOf(json_request(
            self, root, "GET",
            "http://mybase/identity/v2.0/users?name={0}".format(username)
        ))
        self.assertEqual(200, response.code)
        self.assertEqual(json_body['user']['RAX-AUTH:domainId'], "12345")
        self.assertEqual(json_body['user']['username'], username)
        self.assertEqual(json_body['user']['id'], user_id)


class AuthIntegrationTests(SynchronousTestCase):
    """
    Tests that combine multiple auth calls together and assure that they
    return consistent data.
    """

    def test_user_for_tenant_then_impersonation(self):
        """
        After authenticating once as a particular tenant, get the user that
        tenant, then attempt to impersonate that user.  The tenant IDs should
        be the same.  This is an autoscale regression test.
        """
        core, root = core_and_root([ExampleAPI()])
        tenant_id = "111111"

        # authenticate as that user - this is not strictly necessary, since
        # getting a user for a tenant should work regardless of whether a user
        # was previously in the system, but this will ensure that we can check
        # the username
        response, json_body = authenticate_with_username_password(
            self, root, username="my_user", tenant_id=tenant_id)
        self.assertEqual(200, response.code)
        self.assertEqual(tenant_id,
                         json_body['access']['token']['tenant']['id'])

        # get user for tenant
        response, json_body = self.successResultOf(json_request(
            self, root, "GET", "/identity/v1.1/mosso/111111"))
        self.assertEqual(301, response.code)
        user = json_body['user']['id']
        self.assertEqual("my_user", user)

        # impersonate this user
        response, json_body = impersonate_user(self, root, username=user)
        self.assertEqual(200, response.code)
        token = json_body["access"]['token']["id"]

        # get endpoints for this token, see what the tenant is
        response, json_body = self.successResultOf(json_request(
            self, root, "GET",
            "/identity/v2.0/tokens/{0}/endpoints".format(token)))
        self.assertEqual(200, response.code)
        self.assertEqual(tenant_id,
                         json_body["endpoints"][0]["tenantId"])

        # authenticate with this token and see what the tenant is
        response, json_body = authenticate_with_token(
            self, root, token_id=token, tenant_id=tenant_id)
        self.assertEqual(tenant_id,
                         json_body['access']['token']['tenant']['id'])

    def test_api_key_then_other_token_same_tenant(self):
        """
        After authenticating as a particular tenant with an API key,
        authenticate as the same tenant with a token that is different
        from the one returned by the API key response. Both tokens
        should be accessing the same session.
        """
        core, root = core_and_root([ExampleAPI()])
        tenant_id = "123456"

        response, json_body = authenticate_with_api_key(
            self, root, tenant_id=tenant_id)
        self.assertEqual(200, response.code)
        username_from_api_key = json_body["access"]["user"]["name"]

        response, json_body = authenticate_with_token(
            self, root, token_id="fake_111111", tenant_id=tenant_id)
        self.assertEqual(200, response.code)
        username_from_token = json_body["access"]["user"]["name"]

        # Since usernames are generated if not specified, and token
        # authentication does not specify a username, it is sufficient
        # to check that the usernames are equal. If the sessions are
        # distinct, then the token would have generated a UUID for its
        # username.
        self.assertEqual(username_from_api_key, username_from_token)


auth_behavior_endpoint = (
    "http://mybase/mimic/v1.1/IdentityControlAPI/behaviors/auth")


@behavior_tests_helper_class
class IdentityAuthBehaviorControlPlane(object):
    """
    Helper object used to generate tests for Nova create server behavior
    CRUD operations.
    """
    criteria = [{"username": "failme"}]
    names_and_params = (
        ("fail",
         {"message": "Auth failure", "code": 500, "type": "identityFault"}),
        ("fail",
         {"message": "Invalid creds", "code": 403})
    )

    def __init__(self, test_case):
        """
        Set up the criteria, api mock, etc.
        """
        self.test_case = test_case
        _, self.root = core_and_root([])
        self.behavior_api_endpoint = auth_behavior_endpoint

    def trigger_event(self):
        """
        Create server with with the name "failing_server_name".
        """
        return authenticate_with_username_password(
            self.test_case, self.root, username="failme")

    def validate_injected_behavior(self, name_and_params, response, body):
        """
        Given the behavior that is expected, validate the response and body.
        """
        name, params = name_and_params
        self.test_case.assertEquals(response.code, params['code'])
        if params['code'] == 500:
            expected = {"identityFault": {"message": "Auth failure",
                                          "code": 500}}
        else:
            expected = {"unauthorized": {"message": "Invalid creds",
                                         "code": 403}}

        self.test_case.assertEquals(body, expected)

    def validate_default_behavior(self, response, body):
        """
        Validate the response and body of a successful server create.
        """
        self.test_case.assertEquals(response.code, 200)
        self.test_case.assertIn('access', body)


class IdentityBehaviorInjectionTests(SynchronousTestCase):
    """
    Tests for specific failures and/or criteria.
    """
    def test_username_criteria_works_on_all_auth_methods_with_username(self):
        """
        Failure injection based on the username criteria will work on
        username/password, username/api-key, and impersonation.  But not
        token ID, even if it's with the same tenant.
        """
        core, root = core_and_root([])
        fail_params = {"message": "Invalid creds", "code": 403}

        # make sure a user exists in mimic with the given username tenant
        # associated
        response, body = authenticate_with_username_password(
            self, root, username="failme", tenant_id="123456")
        self.assertEqual(response.code, 200)

        # username auths fail
        register_behavior(self, root, auth_behavior_endpoint,
                          behavior_name="fail",
                          criteria=[{"username": "failme"}],
                          parameters=fail_params)
        for auth_func in (authenticate_with_username_password,
                          authenticate_with_api_key,
                          impersonate_user):
            response, body = auth_func(self, root, username="failme")
            self.assertEqual(response.code, 403)
            self.assertEqual(body, {"unauthorized": fail_params})

        # token auth with that tenant ID succeeds
        response, body = authenticate_with_token(
            self, root, tenant_id="123456")
        self.assertEqual(response.code, 200)

    def test_tenant_id_criteria_works_on_all_auth_methods_with_tenant(self):
        """
        Failure injection based on the username criteria will work on
        username/password, username/api-key, and token.
        But not impersonation.
        """
        core, root = core_and_root([])
        fail_params = {"message": "Invalid creds", "code": 403}

        # make sure a user exists in mimic with the given username tenant
        # associated
        response, body = authenticate_with_username_password(
            self, root, username="failme", tenant_id="123456")
        self.assertEqual(response.code, 200)

        # tenant auths fail
        register_behavior(self, root, auth_behavior_endpoint,
                          behavior_name="fail",
                          criteria=[{"tenant_id": "123456"}],
                          parameters=fail_params)
        for auth_func in (authenticate_with_username_password,
                          authenticate_with_api_key,
                          authenticate_with_token):
            response, body = auth_func(self, root, tenant_id="123456")
            self.assertEqual(response.code, 403)
            self.assertEqual(body, {"unauthorized": fail_params})

        # impersonation with that username succeeds
        response, body = impersonate_user(self, root, username="failme")
        self.assertEqual(response.code, 200)

    def test_string_errors_as_well_as_json_errors(self):
        """
        Failure injection will return a string error response as well as a
        json response.
        """
        core, root = core_and_root([])
        fail_params = {"message": "Failure of JSON", "code": 500,
                       "type": "string"}

        register_behavior(self, root, auth_behavior_endpoint,
                          behavior_name="fail",
                          criteria=[{"username": "failme"}],
                          parameters=fail_params)
        response, body = authenticate_with_username_password(
            self, root, username="failme", request_func=request_with_content)
        self.assertEqual(response.code, 500)
        self.assertEqual(body, "Failure of JSON")


class IdentityNondedicatedFixtureTests(SynchronousTestCase):

    def test_non_dedicated_tokens(self):
        """
        Obtain Identity entries when presented tokens issued to non-dedicated users
        """
        url = "/identity/v2.0/tokens"
        core, root = core_and_root([])
        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/OneTwo"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["access"]["token"]["tenant"]["id"], "135790")

        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/ThreeFour"))
        self.assertEqual(200, response.code)


class IdentityDedicatedFixtureTests(SynchronousTestCase):

    def test_dedicated_tokens(self):
        """
        Obtain Identity entries when presented tokens issued to dedicated users
        """
        url = "/identity/v2.0/tokens"
        core, root = core_and_root([])
        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/HybridOneTwo"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["access"]["token"]["tenant"]["id"], "hybrid:123456")
        self.assertEqual(content["access"]["user"]["RAX-AUTH:contactId"], "12")

        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/HybridThreeFour"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["access"]["token"]["tenant"]["id"], "hybrid:123456")
        self.assertEqual(content["access"]["user"]["RAX-AUTH:contactId"], "34")

        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/HybridFiveSix"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["access"]["token"]["tenant"]["id"], "hybrid:123456")
        self.assertEqual(content["access"]["user"]["RAX-AUTH:contactId"], "56")

        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/HybridSevenEight"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["access"]["token"]["tenant"]["id"], "hybrid:654321")
        self.assertEqual(content["access"]["user"]["RAX-AUTH:contactId"], "78")

        (response, content) = self.successResultOf(
            json_request(self, root, "GET", url + "/HybridNineZero"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["access"]["token"]["tenant"]["id"], "hybrid:654321")
        self.assertEqual(content["access"]["user"]["RAX-AUTH:contactId"], "90")



# -*- test-case-name: mimic.test.test_swift -*-

"""
API mock for OpenStack Swift / Rackspace Cloud Files.
"""

from uuid import uuid4, uuid5, NAMESPACE_URL
from six import text_type

from characteristic import attributes, Attribute
from json import dumps

from mimic.imimic import IAPIMock
from twisted.plugin import IPlugin
from twisted.web.http import CREATED, ACCEPTED, OK

from mimic.catalog import Entry
from mimic.catalog import Endpoint
from mimic.rest.mimicapp import MimicApp
from twisted.web.resource import NoResource
from zope.interface import implementer


def normal_tenant_id_to_crazy_mosso_id(normal_tenant_id):
    """
    Convert the tenant ID used by basically everything else (keystone, nova,
    loadbalancers, and so on) into the somewhat peculiar tenant ID used by
    Cloud Files, for maximum realism.

    :return: a new tenant ID that looks like a Cloud Files tenant ID
    :rtype: str
    """
    return (
        "MossoCloudFS_" +
        str(uuid5(NAMESPACE_URL,
                  b"https://github.com/rackerlabs/mimic/ns/tenant/"
                  + normal_tenant_id.encode("utf-8")))
    )


@implementer(IAPIMock, IPlugin)
class SwiftMock(object):
    """
    API mock for Swift.
    """

    def __init__(self, rackspace_flavor=True):
        """
        Construct a SwiftMock, either using Rackspace's tenant-ID translation
        idiom or not.
        """
        if rackspace_flavor:
            self.translate_tenant = normal_tenant_id_to_crazy_mosso_id
        else:
            self.translate_tenant = str

    def catalog_entries(self, tenant_id):
        """
        Catalog entry for Swift endpoints.
        """
        modified = self.translate_tenant(tenant_id)
        return [
            Entry(modified, "object-store", "cloudFiles", [
                Endpoint(modified, "ORD", text_type(uuid4()), prefix="v1"),
            ])
        ]

    def resource_for_region(self, region, uri_prefix, session_store):
        """
        Return an IResource implementing a public Swift region endpoint.
        """
        return SwiftRegion(
            api=self,
            uri_prefix=uri_prefix,
            session_store=session_store).app.resource()


@attributes("api uri_prefix session_store".split())
class SwiftRegion(object):
    """
    :obj:`SwiftRegion` is a set of klein routes and application representing a
    Swift endpoint.
    """

    app = MimicApp()

    @app.route("/v1/<string:tenant_id>", branch=True)
    def get_one_tenant_resource(self, request, tenant_id):
        """
        Get a resource for a tenant in this region.
        """
        return (self.session_store.session_for_tenant_id(tenant_id)
                .data_for_api(self.api,
                              lambda:
                              SwiftTenantInRegion().app.resource()))


@attributes(["name", "content_type", "data"])
class Object(object):
    """
    A Python object (i.e. instance) representing a Swift object (i.e. bag of
    octets).
    """

    def as_json(self):
        """
        Create a JSON-serializable representation of the contents of this
        object.
        """
        return {
            "name": self.name,
            "content_type": self.content_type,
            "bytes": len(self.data),
        }


@attributes(["name", Attribute("objects", default_factory=dict)])
class Container(object):
    """
    A Swift container (collection of :obj:`Object`.)
    """


class SwiftTenantInRegion(object):
    """
    A :obj:`SwiftTenantInRegion` represents a single tenant and their
    associated storage resources within one region.
    """

    app = MimicApp()

    def __init__(self):
        """
        Initialize a tenant with some containers.
        """
        self.containers = {}

    @app.route("/<string:container_name>", methods=["PUT"])
    def create_container(self, request, container_name):
        """
        Api call to create and save container.  HTTP status code of 201 if
        created, else returns 202.
        """
        if container_name not in self.containers:
            self.containers[container_name] = Container(name=container_name)
            request.setResponseCode(CREATED)
        else:
            request.setResponseCode(ACCEPTED)
        return b""

    @app.route("/<string:container_name>", methods=["GET"])
    def get_container(self, request, container_name):
        """
        Api call to get a container, given the name of the container.  HTTP
        status code of 200 when such a container exists, 404 if not.
        """
        if container_name in self.containers:
            request.responseHeaders.setRawHeaders("content-type",
                                                  ["application/json"])
            request.responseHeaders.setRawHeaders("x-container-object-count",
                                                  ["0"])
            request.responseHeaders.setRawHeaders("x-container-bytes-used",
                                                  ["0"])
            request.setResponseCode(OK)
            return dumps([
                obj.as_json() for obj in
                self.containers[container_name].objects.values()
            ])
        else:
            return NoResource()

    @app.route("/<string:container_name>/<string:object_name>",
               methods=["GET"])
    def get_object(self, request, container_name, object_name):
        """
        Get an object from a container.
        """
        return self.containers[container_name].objects[object_name].data

    @app.route("/<string:container_name>/<string:object_name>",
               methods=["PUT"])
    def put_object(self, request, container_name, object_name):
        """
        Create or update an object in a container.
        """
        request.setResponseCode(201)
        container = self.containers[container_name]
        content_type = request.requestHeaders.getRawHeaders('content-type')[0]
        container.objects[object_name] = Object(
            name=object_name, data=request.content.read(),
            content_type=content_type
        )
        return b''



"""
Canned responses for MAAS alarm examples.
"""


def alarm_examples():
    """
    Canned response for the /alarm_examples call.
    """
    return [{
        "id": "remote.http_body_match_1",
        "label": "Body match - string found",
        "description": "Alarm which returns CRITICAL if the provided string is found in the body",
        "check_type": "remote.http",
        "criteria": "if (metric['body_match'] regex '${string}') {\n  return new AlarmStatus(CRITICAL," +
        " '${string} found, returning CRITICAL.');\n}\n",
        "fields": [{
            "name": "string",
            "description": "String to check for in the body",
            "type": "string"
        }]
    }, {
        "id": "remote.http_body_match_missing_string",
        "label": "Body match - string not found",
        "description": "Alarm which returns CRITICAL if the provided string is not found in the body",
        "check_type": "remote.http",
        "criteria": "if (metric['body_match'] == '') {\n  return new AlarmStatus(CRITICAL, 'HTTP " +
        "response did not contain the correct content.');\n}\n\nreturn new AlarmStatus(OK, 'HTTP " +
        "response contains the correct content');\n",
        "fields": []
    }, {
        "id": "remote.http_connection_time",
        "label": "Connection time",
        "description": "Alarm which returns WARNING or CRITICAL based on the connection time",
        "check_type": "remote.http",
        "criteria": "if (metric['duration'] > ${critical_threshold}) {\n  return new AlarmStatus(" +
        "CRITICAL, 'HTTP request took more than ${critical_threshold} milliseconds.');\n}\n\nif (" +
        "metric['duration'] > ${warning_threshold}) {\n  return new AlarmStatus(WARNING, 'HTTP " +
        "request took more than ${warning_threshold} milliseconds.');\n}\n\nreturn new " +
        "AlarmStatus(OK, 'HTTP connection time is normal');\n",
        "fields": [{
            "name": "warning_threshold",
            "description": "Warning threshold (in milliseconds) for the connection time",
            "type": "integer"
        }, {
            "name": "critical_threshold",
            "description": "Critical threshold (in milliseconds) for the connection time",
            "type": "integer"
        }]
    }, {
        "id": "remote.http_status_code",
        "label": "Status code",
        "description": "Alarm which returns WARNING if the server responses with 4xx status code or " +
        "CRITICAL if it responds with 5xx status code",
        "check_type": "remote.http",
        "criteria": "if (metric['code'] regex '4[0-9][0-9]') {\n  return new AlarmStatus(CRITICAL, " +
        "'HTTP server responding with 4xx status');\n}\n\nif (metric['code'] regex '5[0-9][0-9]') {\n" +
        "return new AlarmStatus(CRITICAL, 'HTTP server responding with 5xx status');\n}\n\nreturn " +
        "new AlarmStatus(OK, 'HTTP server is functioning normally');\n",
        "fields": []
    }, {
        "id": "remote.http_cert_expiration",
        "label": "SSL certificate expiration time",
        "description": "Alarm which returns WARNING or CRITICAL based on the certificate expiration " +
        "date",
        "check_type": "remote.http",
        "criteria": "if (metric['cert_end_in'] < ${critical_threshold}) {\n  return new " +
        "AlarmStatus(CRITICAL, 'Cert expiring in less than ${critical_threshold} seconds.');\n}\n\nif " +
        "(metric['cert_end_in'] < ${warning_threshold}) {\n  return new AlarmStatus(WARNING, 'Cert " +
        "expiring in less than ${warning_threshold} seconds.');\n}\n\nreturn new AlarmStatus(OK, " +
        "'HTTP certificate doesn\\'t expire soon.');\n",
        "fields": [{
            "name": "warning_threshold",
            "description": "Warning threshold (in seconds) for the certificate expiration",
            "type": "integer"
        }, {
            "name": "critical_threshold",
            "description": "Critical threshold (in seconds) for the certificate expiration",
            "type": "integer"
        }]
    }, {
        "id": "remote.dns_address_match",
        "label": "DNS record address match",
        "description": "Alarm which returns CRITICAL if the DNS record is not resolved to the " +
        "provided address",
        "check_type": "remote.dns",
        "criteria": "# Match if the 127... address was in the resolution\n# if it wasn't than " +
        "default to CRITICAL\n\nif (metric['answer'] regex '.*${address}.*') {\n  return new " +
        "AlarmStatus(OK, 'Resolved the correct address!');\n}\nreturn new AlarmStatus(CRITICAL);\n",
        "fields": [{
            "name": "address",
            "description": "Address to which the DNS record must resolve to",
            "type": "string"
        }]
    }, {
        "id": "remote.ssh_fingerprint_match",
        "label": "SSH fingerprint match",
        "description": "Alarm which returns CRITICAL if the SSH fingerprint doesn't match the "
        "provided one",
        "check_type": "remote.ssh",
        "criteria": "if (metric['fingerprint'] != '${fingerprint}') {\n  return new " +
        "AlarmStatus(CRITICAL, 'SSH fingerprint didn\\'t match the expected one ${fingerprint}');" +
        "\n}\n\nreturn new AlarmStatus(OK, 'Got expected SSH fingerprint (${fingerprint})');\n",
        "fields": [{
            "name": "fingerprint",
            "description": "Expected SSH fingerprint",
            "type": "string"
        }]
    }, {
        "id": "remote.ping_packet_loss",
        "label": "Ping packet loss",
        "description": "Alarm which returns WARNING if the packet loss is greater than 5% and " +
        "CRITICAL if it's greater than 20%",
        "check_type": "remote.ping",
        "criteria": "if (metric['available'] < 80) {\n  return new AlarmStatus(CRITICAL, 'Packet loss" +
        " is greater than 20%');\n}\n\nif (metric['available'] < 95) {\n  return new " +
        "AlarmStatus(WARNING, 'Packet loss is greater than 5%');\n}\n\nreturn new AlarmStatus(OK, " +
        "'Packet loss is normal');\n",
        "fields": []
    }, {
        "id": "remote.tcp_connection_time",
        "label": "Connection time",
        "description": "Alarm which returns WARNING or CRITICAL based on the connection time",
        "check_type": "remote.tcp",
        "criteria": "if (metric['duration'] > ${critical_threshold}) {\n  return new AlarmStatus(" +
        "CRITICAL, 'TCP Connection took more than ${critical_threshold} milliseconds.');\n}\n\nif " +
        "(metric['duration'] > ${warning_threshold}) {\n  return new AlarmStatus(WARNING, 'TCP " +
        "Connection took more than ${warning_threshold} milliseconds.');\n}\n\nreturn new " +
        "AlarmStatus(OK, 'TCP connection time is normal');\n",
        "fields": [{
            "name": "warning_threshold",
            "description": "Warning threshold (in seconds) for the connection time",
            "type": "integer"
        }, {
            "name": "critical_threshold",
            "description": "Critical threshold (in seconds) for the connection time",
            "type": "integer"
        }]
    }, {
        "id": "remote.dns_spf_record_include_match",
        "label": "SPF TXT record",
        "description": "Alarm which returns CRITICAL if the SPF record doesn't contain an include " +
        "clause with the provided domain.",
        "check_type": "remote.dns",
        "criteria": "if (metric['answer'] regex 'v=spf1.* include:${domain} .*[~|-]all') {\n  return " +
        "new AlarmStatus(OK, 'SPF record with include clause for domain ${domain} exists');\n}\n\n" +
        "return new AlarmStatus(CRITICAL, 'SPF record doesn\\'t contain an include clause for domain " +
        "${domain}');\n",
        "fields": [{
            "name": "domain",
            "description": "Domain to check for",
            "type": "string"
        }]
    }, {
        "id": "remote.dns_dkim_public_key_match",
        "label": "DKIM TXT record",
        "description": "Alarm which returns CRITICAL if the DKIM record doesn't contain or match the " +
        "provided public key.",
        "check_type": "remote.dns",
        "criteria": "if (metric['answer'] regex '.*p=${public_key}$') {\n  return new AlarmStatus(OK," +
        " 'DKIM record contains a provided public key');\n}\n\nreturn new AlarmStatus(CRITICAL, 'DKIM " +
        "record doesn\\'t contain a provided public key');\n",
        "fields": [{
            "name": "public_key",
            "description": "Public key to check for. Note: Special characters must be escaped.",
            "type": "string"
        }]
    }, {
        "id": "agent.cpu_usage_average",
        "label": "CPU Usage",
        "description": "Alarm which returns CRITICAL, WARNING or OK based upon average CPU usage",
        "check_type": "agent.cpu",
        "criteria": "if (metric['usage_average'] > ${critical_threshold}) {\n  return new " +
        "AlarmStatus(CRITICAL, 'CPU usage is #{usage_average}%, above your critical threshold of " +
        "${critical_threshold}%');\n}\n\nif (metric['usage_average'] > ${warning_threshold}) {\n  " +
        "return new AlarmStatus(WARNING, 'CPU usage is #{usage_average}%, above your warning threshold" +
        " of ${warning_threshold}%');\n}\n\nreturn new AlarmStatus(OK, 'CPU usage is " +
        "#{usage_average}%, below your warning threshold of ${warning_threshold}%');\n",
        "fields": [{
            "name": "critical_threshold",
            "description": "CPU usage percentage above which CRITICAL is returned",
            "type": "whole number (may be zero padded)"
        }, {
            "name": "warning_threshold",
            "description": "CPU usage percentage above which WARNING is returned",
            "type": "whole number (may be zero padded)"
        }]
    }, {
        "id": "agent.memory_usage",
        "label": "Memory usage",
        "description": "Alarm which returns CRITICAL, WARNING or OK based upon memory usage",
        "check_type": "agent.memory",
        "criteria": "if (percentage(metric['actual_used'], metric['total']) > 90) {\n  return new " +
        "AlarmStatus(CRITICAL, \"Memory usage is above your critical threshold of 90%\");\n}\n\nif " +
        "(percentage(metric['actual_used'], metric['total']) > 80) {\n  return new AlarmStatus(" +
        "WARNING, \"Memory usage is above your warning threshold of 80%\");\n}\n\nreturn new " +
        "AlarmStatus(OK, \"Memory usage is below your warning threshold of 80%\");\n",
        "fields": []
    }, {
        "id": "agent.filesystem_usage",
        "label": "Filesystem usage",
        "description": "Alarm which returns CRITICAL, WARNING or OK based upon filesystem usage",
        "check_type": "agent.filesystem",
        "criteria": "if (percentage(metric['used'], metric['total']) > 90) {\n  return new " +
        "AlarmStatus(CRITICAL, \"Disk usage is above your critical threshold of 90%\");\n}\n\nif " +
        "(percentage(metric['used'], metric['total']) > 80) {\n  return new AlarmStatus(WARNING, " +
        "\"Disk usage is above your warning threshold of 80%\");\n}\n\nreturn new AlarmStatus(OK, " +
        "\"Disk usage is below your warning threshold of 80%\");\n",
        "fields": []
    }, {
        "id": "agent.high_load_average",
        "label": "High Load Average",
        "description": "Alarm which returns CRITICAL, WARNING or OK based on load average",
        "check_type": "agent.load_average",
        "criteria": "if (metric['5m'] > ${critical_threshold}) {\n  return new AlarmStatus(CRITICAL," +
        " '5 minute load average is #{5m}, above your critical threshold of ${critical_threshold}');" +
        "\n}\n\nif (metric['5m'] > ${warning_threshold}) {\n  return new AlarmStatus(WARNING, '5 " +
        "minute load average is #{5m}, above your warning threshold of ${warning_threshold}');\n}\n\n" +
        "return new AlarmStatus(OK, '5 minute load average is #{5m}, below your warning threshold of " +
        "${warning_threshold}');\n",
        "fields": [{
            "name": "critical_threshold",
            "description": "Load average above which CRITICAL is returned",
            "type": "whole number (may be zero padded)"
        }, {
            "name": "warning_threshold",
            "description": "Load average above which WARNING is returned",
            "type": "whole number (may be zero padded)"
        }]
    }, {
        "id": "agent.network_transmit_rate",
        "label": "Network transmit rate",
        "description": "Alarm which returns CRITICAL, WARNING or OK based upon network transmit rate",
        "check_type": "agent.network",
        "criteria": "if (rate(metric['tx_bytes']) > ${critical_threshold}) {\n  return new " +
        "AlarmStatus(CRITICAL, \"Network transmit rate on ${interface} is above your critical " +
        "threshold of ${critical_threshold}B/s\");\n}\n\nif (rate(metric['tx_bytes']) > " +
        "${warning_threshold}) {\n  return new AlarmStatus(WARNING, \"Network transmit rate on " +
        "${interface} is above your warning threshold of ${warning_threshold}B/s\");\n}\n\nreturn new " +
        "AlarmStatus(OK, \"Network transmit rate on ${interface} is below your warning threshold of " +
        "${warning_threshold}B/s\");\n",
        "fields": [{
            "name": "interface",
            "description": "The network interface to alert on",
            "type": "string"
        }, {
            "name": "critical_threshold",
            "description": "Network transmit rate, in bytes per second, above which CRITICAL is " +
            "returned",
            "type": "whole number (may be zero padded)"
        }, {
            "name": "warning_threshold",
            "description": "Network transmit rate, in bytes per second, above which WARNING is returned",
            "type": "whole number (may be zero padded)"
        }]
    }, {
        "id": "agent.network_receive_rate",
        "label": "Network receive rate",
        "description": "Alarm which returns CRITICAL, WARNING or OK based upon network receive rate",
        "check_type": "agent.network",
        "criteria": "if (rate(metric['rx_bytes']) > ${critical_threshold}) {\n  return new " +
        "AlarmStatus(CRITICAL, \"Network receive rate on ${interface} is above your critical " +
        "threshold of ${critical_threshold}B/s\");\n}\n\nif (rate(metric['rx_bytes']) > " +
        "${warning_threshold}) {\n  return new AlarmStatus(WARNING, \"Network receive rate on " +
        "${interface} is above your warning threshold of ${warning_threshold}B/s\");\n}\n\nreturn new " +
        "AlarmStatus(OK, \"Network receive rate on ${interface} is below your warning threshold of " +
        "${warning_threshold}B/s\");\n",
        "fields": [{
            "name": "interface",
            "description": "The network interface to alert on",
            "type": "string"
        }, {
            "name": "critical_threshold",
            "description": "Network receive rate, in bytes per second, above which CRITICAL is returned",
            "type": "whole number (may be zero padded)"
        }, {
            "name": "warning_threshold",
            "description": "Network receive rate, in bytes per second, above which WARNING is returned",
            "type": "whole number (may be zero padded)"
        }]
    }, {
        "id": "agent.mysql_threads_connected_threshold",
        "label": "Connected Threads",
        "description": "Alarm which returns WARNING if the threads connected is over the specified " +
        "threshold and OK if it is under the specified threshold.",
        "check_type": "agent.mysql",
        "criteria": "if (metric['threads.connected'] > ${threads.connected.threshold}) {\n\treturn " +
        "new AlarmStatus(WARNING, 'Total number of threads connected are above your threshold of " +
        "${threads.connected.threshold}');\n}\nreturn new AlarmStatus(OK, 'Total number of threads " +
        "connected are below your warning threshold of ${threads.connected.threshold}');\n",
        "fields": [{
            "name": "threads.connected.threshold",
            "description": "Warning threshold for the number of connections",
            "type": "whole number (may be zero padded)"
        }]
    }]



"""
Resources for Mimic's core application.
"""
import json

from io import StringIO

from six import text_type

from twisted.web.resource import NoResource
from twisted.web.server import Request, Site

# Just import from twisted.logger if mimic drops support for twisted < 15.2.0
try:
    from twisted.logger import Logger
except ImportError:
    from twisted.python.log import msg

    def log(message, **kwargs):
        """
        PEP3101-format the message string.
        """
        msg(message.format(**kwargs))
else:
    log = Logger("mimic").info

from mimic.canned_responses.mimic_presets import get_presets
from mimic.model.behaviors import BehaviorRegistryCollection
from mimic.rest.mimicapp import MimicApp
from mimic.rest.auth_api import (
    AuthApi,
    AuthControlApiBehaviors,
    base_uri_from_request
)
from mimic.rest.noit_api import NoitApi
from mimic.rest import (fastly_api, mailgun_api, customer_api,
                        ironic_api, glance_api, valkyrie_api)
from mimic.util.helper import seconds_to_timestamp


class MimicRoot(object):
    """
    Klein routes for the root of the mimic URI hierarchy.
    """

    app = MimicApp()

    def __init__(self, core, clock=None):
        """
        :param mimic.core.MimicCore core: The core object to dispatch routes
            from.
        :param twisted.internet.task.Clock clock: The clock to advance from the
            ``/mimic/v1.1/tick`` API.
        """
        self.core = core
        self.clock = clock
        self.identity_behavior_registry = BehaviorRegistryCollection()

    @app.route("/", methods=["GET"])
    def help(self, request):
        """
        A helpful greeting message.
        """
        request.responseHeaders.setRawHeaders("content-type", ["text/plain"])
        return ("To get started with Mimic, POST an authentication request to:"
                "\n\n/identity/v2.0/tokens\n")

    @app.route("/identity", branch=True)
    def get_auth_api(self, request):
        """
        Get the identity ...
        """
        return AuthApi(self.core,
                       self.identity_behavior_registry).app.resource()

    @app.route("/noit", branch=True)
    def get_noit_api(self, request):
        """
        Mock Noit api here ... until mimic allows services outside of the
        service catalog.
        """
        return NoitApi(self.core, self.clock).app.resource()

    @app.route("/sendgrid/mail.send.json", methods=['POST'])
    def send_grid_api(self, request):
        """
        Mock SendGrid api responds with a 200.
        """
        request.setResponseCode(200)
        return b''

    @app.route("/cloudmonitoring.rackspace.com", branch=True)
    def mailgun_api(self, request):
        """
        Mock Mail Gun API.
        """
        return mailgun_api.MailGunApi(self.core).app.resource()

    @app.route("/fastly", branch=True)
    def get_fastly_api(self, request):
        """
        Get the Fastly API ...
        """
        return fastly_api.FastlyApi(self.core).app.resource()

    @app.route("/v1/customer_accounts/CLOUD", branch=True)
    def get_customer_api(self, request):
        """
        Adds support for the Customer API
        """
        return customer_api.CustomerApi(self.core).app.resource()

    @app.route("/ironic/v1", branch=True)
    def ironic_api(self, request):
        """
        Mock Ironic API.
        """
        return ironic_api.IronicApi(self.core).app.resource()

    @app.route("/valkyrie/v2.0", branch=True)
    def valkyrie_api(self, request):
        """
        Mock Valkyrie API.
        """
        return valkyrie_api.ValkyrieApi(self.core).app.resource()

    @app.route('/mimic/v1.0/presets', methods=['GET'])
    def get_mimic_presets(self, request):
        """
        Return the preset values for mimic
        """
        request.setResponseCode(200)
        return json.dumps(get_presets)

    @app.route("/mimic/v1.1/tick", methods=['POST'])
    def advance_time(self, request):
        """
        Advance time by the given number of seconds.
        """
        body = json.loads(request.content.read())
        amount = body['amount']
        self.clock.advance(amount)
        request.setResponseCode(200)
        return json.dumps({
            "advanced": amount,
            "now": seconds_to_timestamp(self.clock.seconds())
        })

    @app.route("/mimic/v1.1/IdentityControlAPI/behaviors", branch=True)
    def handle_identity_behaviors(self, request):
        """
        Handle creating/deleting behaviors for mimic identity.
        """
        api = AuthControlApiBehaviors(self.identity_behavior_registry)
        return api.app.resource()

    @app.route("/mimicking/<string:service_id>/<string:region_name>",
               branch=True)
    def get_service_resource(self, request, service_id, region_name):
        """
        Based on the URL prefix of a region and a service, where the region is
        an identifier (like ORD, DFW, etc) and service is a
        dynamically-generated UUID for a particular plugin, retrieve the
        resource associated with that service.
        """
        service_object = self.core.service_with_region(
            region_name, service_id, base_uri_from_request(request))

        if service_object is None:
            # workaround for https://github.com/twisted/klein/issues/56
            return NoResource()
        return service_object

    @app.route("/glance", branch=True)
    def glance_admin_api(self, request):
        """
        Mock for the glance admin api
        """
        return glance_api.GlanceAdminApi(self.core).app.resource()


class MimicRequest(Request, object):
    """
    Mimic requests by default are of content type application/json.
    """
    defaultContentType = "application/json"


class MimicLoggingRequest(MimicRequest, object):
    """
    Mimic request that by default logs all incoming requests and outgoing
    responses.
    """

    def __init__(self, *args, **kwargs):
        """
        Same as the superclass's :obj:`__init__` except it also creates a
        buffer to store the response for logging.
        """
        super(MimicLoggingRequest, self).__init__(*args, **kwargs)
        self.response_body_for_logging = StringIO()

    def process(self):
        """
        Log the incoming response before calling the superclass's
        :obj:`process`.
        """
        content = self.content.read()
        self.content.seek(0)
        log("Received request: {method} {url}\n"
            "Headers: {headers}\n"
            "{body}",
            method=self.method, url=self.uri,
            headers=json.dumps(dict(self.requestHeaders.getAllRawHeaders())),
            body=("\n" + content + "\n" if content else ""))
        return super(MimicLoggingRequest, self).process()

    def write(self, data):
        """
        Collect the response data before calling the superclass's :obj:`write`.
        """
        self.response_body_for_logging.write(text_type(data))
        return super(MimicLoggingRequest, self).write(data)

    def finish(self):
        """
        Before finishing the request, log the response.
        """
        content = self.response_body_for_logging.getvalue()
        log("Responding with {code} for: {method} {url}\n"
            "Headers: {headers}\n"
            "{body}",
            method=self.method, url=self.uri, code=self.code,
            headers=json.dumps(dict(self.responseHeaders.getAllRawHeaders())),
            body=("\n" + content + "\n" if content else ""))
        return super(MimicLoggingRequest, self).finish()


def get_site(resource, logging=False):
    """
    :param resource: A :class:`twisted.web.resource.Resource` object.
    :return: a :class:`Site` that can be run
    """
    site = Site(resource)
    site.displayTracebacks = False
    site.requestFactory = MimicLoggingRequest if logging else MimicRequest
    return site



import urllib

from twisted.trial.unittest import SynchronousTestCase
from twisted.internet.task import Clock

from mimic.core import MimicCore
from mimic.resource import MimicRoot
from mimic.test.helpers import json_request, request


class MailGunAPITests(SynchronousTestCase):

    """
    Tests for the Mailgun api
    """

    def setUp(self):
        """
        Initialize core and root
        """
        self.core = MimicCore(Clock(), [])
        self.root = MimicRoot(self.core).app.resource()

    def create_message_successfully(self, root, data):
        """
        Create a message and validate the response is the
        """
        (response, content) = self.successResultOf(json_request(
            self, root, "POST", "/cloudmonitoring.rackspace.com/messages",
            urllib.urlencode(data)))
        self.assertEqual(200, response.code)

    def get_content_from_list_messages(self, root, to_filter=None):
        """
        Get messages and return content
        """
        url = "/cloudmonitoring.rackspace.com/messages"
        if to_filter:
            url = "/cloudmonitoring.rackspace.com/messages?to={0}".format(to_filter)
        (response, content) = self.successResultOf(json_request(
            self, root, "GET", url))
        self.assertEqual(200, response.code)
        return content

    def test_mailgun_send_message(self):
        """
        ``/cloudmonitoring.rackspace.com/messages`` returns response code 200
        when a create message is successful.
        """
        self.create_message_successfully(
            self.root,
            {"to": "example@eg.com", "subject": "test"})

    def test_mailgun_send_message_receives_error_500(self):
        """
        ``/cloudmonitoring.rackspace.com/messages`` returns response code 500
        when the `to` address is `bademail@example.com`.
        """
        response = self.successResultOf(request(
            self, self.root, "POST", "/cloudmonitoring.rackspace.com/messages",
            urllib.urlencode({"to": "bademail@example.com"})))
        self.assertEqual(500, response.code)

    def test_mailgun_send_message_receives_error_400(self):
        """
        ``/cloudmonitoring.rackspace.com/messages`` returns response code 400
        when the `to` address is `failingemail@example.com`.
        """
        response = self.successResultOf(request(
            self, self.root, "POST", "/cloudmonitoring.rackspace.com/messages",
            urllib.urlencode({"to": "failingemail@example.com"})))
        self.assertEqual(400, response.code)

    def test_mailgun_get_messages(self):
        """
        ``/cloudmonitoring.rackspace.com/messages`` returns response code 200
        and returns the list of messages created thus far.
        """
        for x in range(5):
            self.create_message_successfully(
                self.root,
                {"to": "example{0}@eg.com".format(x), "subject": "test"})
        content = self.get_content_from_list_messages(self.root)
        self.assertEqual(len(content["items"]), 5)

    def test_mailgun_get_messages_by_filter(self):
        """
        ``/cloudmonitoring.rackspace.com/messages`` returns response code 200
        and returns the list of messages created thus far, fitered by `to`
        address.
        """
        for x in range(5):
            self.create_message_successfully(
                self.root,
                {"to": "example{0}@eg.com".format(x), "subject": "test"})
        content = self.get_content_from_list_messages(self.root, 'example0@eg.com')
        self.assertEqual(len(content["items"]), 1)

    def test_mailgun_get_messages_resulting_in_500s(self):
        """
        ``/cloudmonitoring.rackspace.com/messages/500s`` returns response code 200
        and count of requests that resulted in 500s during message creation.
        """
        for x in range(5):
            response = self.successResultOf(request(
                self, self.root, "POST", "/cloudmonitoring.rackspace.com/messages",
                urllib.urlencode({"to": "bademail@example.com", "subject": "test"})))
            self.assertEqual(500, response.code)

        (response, content) = self.successResultOf(json_request(
            self, self.root, "GET", "/cloudmonitoring.rackspace.com/messages/500s"))
        self.assertEqual(200, response.code)
        self.assertEqual(content["count"], 5)

    def test_mailgun_get_message_header_for_message(self):
        """
        ``/cloudmonitoring.rackspace.com/messages/headers`` returns response code 200
        and headers recieved for a given `to` address.
        """
        self.create_message_successfully(
            self.root,
            {"to": "email@example.com", "h:X-State": ["WARNING"],
             "subject": "test"})

        (response, content) = self.successResultOf(json_request(
            self, self.root,
            "GET", "/cloudmonitoring.rackspace.com/messages/headers?to=email@example.com"))
        self.assertEqual(200, response.code)
        self.assertTrue(content["email@example.com"])



"""
A pseudo enumeration of errors seen from Rackspace Cloud Load Balancers.

Note: This is not a good way to make these available to users to specify, and
so the API will probably change.  But this seems better than sprinkling them
all over the request code right now.
"""


def considered_immutable_error(clb_state, lb_id):
    """
    Return the error seen whenever the load balancer is not in an ACTIVE state,
    in which case it is immutable.  Any modifications to the load balancer
    (adding nodes, deleting nodes, changing nodes) will result in this
    message.

    Not sure if this message is applicable when deleting the load balancer.

    :param str clb_state: The states in which the CLB is considered immutable.
        The state should be one of (PENDING_DELETE, PENDING_UPDATE, ERROR).
        This state gets incorporated into the message.
    :param str lb_id: The load balancer ID, which gets incorporated into the
        message.

    :return: a `tuple` of (dict body message, 422 http status code)
    """
    return (
        {
            "message": "Load Balancer '{0}' has a status of '{1}' and is "
                       "considered immutable.".format(lb_id, clb_state),
            "code": 422
        },
        422)


def updating_node_validation_error(address=False, port=False, id=False,
                                   weight=False):
    """
    Verified 2015-06-16:

    - when trying to update a CLB node's address/port/id, which are
      immutable.
    - when trying to update a CLB node's weight to be < 1 or > 100

    At least one of address, port, id, and weight should be `True` for this
    error to apply.

    :param bool address: Whether the address was passed to update
    :param bool port: Whether the port was passed to update
    :param bool id: Whether the ID was passed to update
    :param bool weight: Whether the weight was passed to update and wrong

    :return: a `tuple` of (dict body message, 400 http status code)
    """
    messages = []
    if address:
        messages.append("Node ip field cannot be modified.")
    if port:
        messages.append("Port field cannot be modified.")
    if weight:
        messages.append("Node weight is invalid. Range is 1-100. "
                        "Please specify a valid weight.")
    if id:
        messages.append("Node id field cannot be modified.")

    return(
        {
            "validationErrors": {
                "messages": messages
            },
            "message": "Validation Failure",
            "code": 400,
            "details": "The object is not valid"
        },
        400
    )


def invalid_json_schema():
    """
    Verified 2015-06-16 for creating a LB, changing a node, adding nodes:

    - when providing invalid fields in the JSON request
    - when providing invalid JSON
    - this schema validation happens even before checking whether a LB and/or
      node exists

    :return: a `tuple` of (dict body message, 400 http status code)
    """
    return(
        {
            "message": ["JSON does not match the expected schema"]
        },
        400
    )


def node_not_found():
    """
    Verified 2015-06-16 for updating a node that does not exist.

    :return: a `tuple` of (dict body message, 404 http status code)
    """
    return (
        {
            "message": "Node not found",
            "code": 404
        },
        404
    )


def loadbalancer_not_found():
    """
    Verified 2015-06-16:
    - when trying to update a CLB node on a CLB that doesn't exist

    :return: a `tuple` of (dict body message, 404 http status code)
    """
    return (
        {
            "message": "Load balancer not found",
            "code": 404
        },
        404
    )


def not_found_xml(item):
    """
    Return XML representation of CLB item not found.
    """
    return (
        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
        '<itemNotFound xmlns="http://docs.openstack.org/loadbalancers/api/v1.0" code="404">'
        '<message>{0} not found</message></itemNotFound>').format(item), 404


def lb_deleted_xml():
    """
    Return XML representation of CLB marked as DELETED
    """
    return (
        '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
        '<message xmlns="http://docs.openstack.org/loadbalancers/api/v1.0" code="410">'
        'The loadbalancer is marked as deleted.</message>'), 410



import json
from uuid import uuid4

from twisted.trial.unittest import SynchronousTestCase
from twisted.internet.task import Clock

from mimic.core import MimicCore
from mimic.resource import MimicRoot
from mimic.test.helpers import json_request, request
from mimic.model.ironic_objects import IronicNodeStore, Node


class IronicNodeStoreUnitTests(SynchronousTestCase):
    """
    Unit test for `node_by_id` in :obj:`IronicNodeStore`
    """

    def test_node_by_id(self):
        """
        Test for :func:`node_by_id`
        """
        node_list = [Node(node_id=str(uuid4())) for each in range(2)]
        node_store = IronicNodeStore(ironic_node_store=node_list)
        self.assertFalse(node_store.node_by_id('111'))


class IronicAPITests(SynchronousTestCase):

    """
    Tests for the Ironic api
    """

    def setUp(self):
        """
        Initialize core and root
        """
        self.core = MimicCore(Clock(), [])
        self.root = MimicRoot(self.core).app.resource()
        self.node_details_attributes = [
            "instance_uuid", "target_power_state", "chassis_uuid",
            "properties", "uuid", "driver_info", "target_provision_state",
            "last_error", "console_enabled", "extra", "driver", "links",
            "maintenance_reason", "updated_at", "provision_updated_at",
            "maintenance", "provision_state", "reservation", "created_at",
            "power_state", "instance_info", "ports", "name", "driver_internal_info",
            "inspection_finished_at", "inspection_started_at", "clean_step"
        ]
        self.url = "/ironic/v1/nodes"
        self.create_request = {
            "chassis_uuid": str(uuid4()),
            "driver": "agent_ipmitool",
            "driver_info": {"ipmi_username": "mimic-user",
                            "ipmi_address": "127.0.0.0",
                            "ipmi_password": "******"},
            "name": "test_node",
            "properties": {
                "cpus": "1",
                "local_gb": "10",
                "memory_mb": "1024"
            }
        }

    def create_node(self, create_request=None):
        """
        Create a new node and return node.
        """
        request = create_request or self.create_request
        (response, content) = self.successResultOf(json_request(
            self, self.root, "POST", self.url, body=json.dumps(request)))
        self.assertEqual(response.code, 201)
        return content

    def get_nodes(self, postfix=None):
        """
        Get nodes and return content
        """
        url = self.url
        if postfix:
            url = self.url + postfix
        (response, content) = self.successResultOf(json_request(
            self, self.root, "GET", url))
        self.assertEqual(200, response.code)
        return content

    def test_create_node(self):
        """
        Create node returns 201 and the newly created node.
        """
        new_node = self.create_node(self.create_request)
        for key in self.create_request.keys():
            self.assertEqual(self.create_request[key], new_node[key])

    def test_create_node_failure(self):
        """
        Create node returns 400 if the request json is not
        as expected.
        """
        response = self.successResultOf(request(
            self, self.root, "POST", self.url, body=json.dumps("")))
        self.assertEqual(response.code, 400)

    def test_delete_node_when_node_does_not_exist(self):
        """
        Delete node returns 404 when the given node_id
        does not exist.
        """
        response = self.successResultOf(request(
            self, self.root, "DELETE", self.url + "/1234"))
        self.assertEqual(response.code, 404)

    def test_delete_node(self):
        """
        Delete node returns 204 deletes the given node_id.
        """
        content = self.create_node()
        node_id = str(content['uuid'])

        # delete node
        response = self.successResultOf(request(
            self, self.root, "DELETE", self.url + '/' + node_id))
        self.assertEqual(response.code, 204)

        # get node
        response = self.successResultOf(request(
            self, self.root, "GET", self.url + '/' + node_id))
        self.assertEqual(404, response.code)

    def test_create_then_get_node(self):
        """
        Test create node then get the node and verify attributes
        """
        create_request = {"properties": {"memory_mb": 32768}}
        content = self.create_node(create_request)
        node_id = str(content['uuid'])
        self.assertEqual(content['properties']['memory_mb'],
                         create_request['properties']['memory_mb'])

        # get node
        (response, get_content) = self.successResultOf(json_request(
            self, self.root, "GET", self.url + '/' + node_id))
        self.assertEqual(200, response.code)
        self.assertEqual(content, get_content)

    def test_create_then_get_node_with_default_attributes(self):
        """
        Test create node then get the node and verify attributes
        """
        content = self.create_node()
        node_id = str(content['uuid'])

        # get node
        (response, get_content) = self.successResultOf(json_request(
            self, self.root, "GET", self.url + '/' + node_id))
        self.assertEqual(200, response.code)
        self.assertEqual(content, get_content)

    def test_create_then_get_node_with_mimimum_attributes(self):
        """
        Test create node then get the node and verify attributes
        """
        (response, content) = self.successResultOf(json_request(
            self, self.root, "POST", self.url, body=json.dumps({})))
        self.assertEqual(response.code, 201)
        node_id = str(content['uuid'])
        self.assertFalse(content['properties']['memory_mb'])

        # get node
        (response, get_content) = self.successResultOf(json_request(
            self, self.root, "GET", self.url + '/' + node_id))
        self.assertEqual(200, response.code)
        self.assertEqual(content, get_content)

    def test_create_then_get_node_with_empty_properties_attributes(self):
        """
        Test create node when `properties` attribute is set to None
        """
        self.create_node({"properties": None})

    def test_list_nodes(self):
        """
        Test ``/nodes`` to return response code 200 and validate the
        attributes of the json response body
        """
        content = self.get_nodes()
        for each_node in content['nodes']:
            self.assertEqual(
                sorted(each_node.keys()),
                sorted(['instance_uuid', 'uuid', 'links', 'maintenance',
                        'provision_state', 'power_state', 'name']))

    def test_list_nodes_with_details(self):
        """
        Test ``/nodes/detail`` to return response code 200 and validate a
        response body exists.
        """
        content = self.get_nodes('/detail')
        self.assertTrue(len(content['nodes']) > 1)
        for each_node in content['nodes']:
            self.assertEqual(
                sorted(each_node.keys()),
                sorted(self.node_details_attributes))
        instance_nodes = [node['extra']['flavor'] if node['extra'].get('flavor')
                          else None
                          for node in content['nodes']]
        self.assertEqual(instance_nodes.count('onmetal-io1'), 30)
        self.assertEqual(instance_nodes.count('onmetal-compute1'), 32)
        self.assertEqual(instance_nodes.count('onmetal-memory1'), 30)
        provisioned = [node for node in content['nodes']
                       if node['instance_uuid']]
        self.assertEqual(len(provisioned), 2)

    def test_list_nodes_with_details_is_consistent(self):
        """
        Test ``/nodes/detail`` to return response code 200 and validate a
        response body is the consistent.
        """
        content1 = self.get_nodes('/detail')
        content2 = self.get_nodes('/detail')
        self.assertTrue(content1, content2)

    def test_get_node_details(self):
        """
        Test ``/nodes/<node_id>`` to return response code 200 and validate the
        attributes of the json response body
        """
        # create a node
        content = self.create_node()
        node_id = content['uuid']

        # get node
        content = self.get_nodes('/' + str(node_id))
        self.assertEqual(
            sorted(content.keys()),
            sorted(self.node_details_attributes))

    def test_list_node_details_for_onmetal_flavors(self):
        """
        Test ``/nodes/detail`` to return response code 200 and validate the
        onmetal flavors have the corresponding memory associated on a node.
        """
        expected_flavor_memory = {"onmetal-io1": 131072, "onmetal-compute1": 32768,
                                  "onmetal-memory1": 524288}
        content = self.get_nodes('/detail')
        for each in content['nodes']:
            if each['properties']['memory_mb']:
                self.assertTrue(
                    (each['extra']['flavor'] in expected_flavor_memory.keys()) and
                    (each['properties']['memory_mb'] == expected_flavor_memory[each['extra']['flavor']]))

    def _validate_provisioning(self, new_provision_state):
        """
        Creates a node and verifies the node is 'available'.
        Changes the provision_state of the nodes to `new_provision_state`
        and verifies the state is set.
        Returns the node object after the update.
        """
        # create a node
        content = self.create_node()
        node_id = content['uuid']
        provision_state = content['provision_state']
        self.assertEqual(provision_state, 'available')

        # Change the provision_state
        url = self.url + "/{0}/states/provision".format(node_id)
        response = self.successResultOf(request(
            self, self.root, "PUT", url, body=json.dumps(
                {'target': new_provision_state})))
        self.assertEqual(response.code, 202)

        content = self.get_nodes('/' + str(node_id))
        return content

    def test_setting_provision_state_to_manage(self):
        """
        Test ``/nodes/<node-id>/states/provision`` returns a 200 and
        sets the `provision_state` to 'manage' on the node.
        """
        content = self._validate_provisioning('manage')
        self.assertEqual(content['provision_state'], 'manage')
        self.assertFalse(content['driver_info'].get('cache_image_id'))
        self.assertFalse(content['driver_info'].get('cache_status'))

    def test_setting_provision_state_to_provide(self):
        """
        Test ``/nodes/<node-id>/states/provision`` returns a 200 and
        sets the `provision_state` to 'available' on the node has
        provision_state set to 'provide'
        """
        content = self._validate_provisioning('provide')
        self.assertEqual(content['provision_state'], 'available')
        self.assertFalse(content['driver_info'].get('cache_image_id'))
        self.assertFalse(content['driver_info'].get('cache_status'))

    def test_setting_provision_state_to_active(self):
        """
        Test ``/nodes/<node-id>/states/provision`` returns a 200 and
        sets the `provision_state` to 'active' on the node and
        assigns a instance id.
        """
        content = self._validate_provisioning('active')
        self.assertEqual(content['provision_state'], 'active')
        self.assertTrue(content['instance_uuid'])

    def test_setting_provision_state_fails(self):
        """
        Test ``/nodes/<node-id>/states/provision`` returns a 404 when
        the node does not exist
        """
        url = self.url + "/111/states/provision"
        (response, content) = self.successResultOf(json_request(
            self, self.root, "PUT", url, body=json.dumps({'target': 'active'})))
        self.assertEqual(response.code, 404)
        self.assertTrue('111' in content['error_message']['faultstring'])

    def test_vendor_passthru_cache_image_fails_when_node_not_found(self):
        """
        Test ``/nodes/<node-id>/vendor_passthru/cache_image`` returns a 404 when
        the node does not exist
        """
        url = self.url + "/222/vendor_passthru/cache_image"
        (response, content) = self.successResultOf(json_request(
            self, self.root, "POST", url, body=json.dumps({})))
        self.assertEqual(response.code, 404)
        self.assertTrue('222' in content['error_message']['faultstring'])

    def test_vendor_passthru_cache_image_fails_when_method_not_found(self):
        """
        Test ``/nodes/<node-id>/vendor_passthru/cache_image`` returns a 400 when
        the the method is not `cache_image`
        """
        new_node = self.create_node()
        node_id = new_node['uuid']

        url = self.url + "/{0}/vendor_passthru/not_cache_image".format(node_id)
        response = self.successResultOf(request(
            self, self.root, "POST", url, body=json.dumps({})))
        self.assertEqual(response.code, 400)

    def test_vendor_passthru_cache_image_fails_when_args_invalid(self):
        """
        Test ``/nodes/<node-id>/vendor_passthru/cache_image`` returns a 400 when
        the body for the request is invalid
        """
        new_node = self.create_node()
        node_id = new_node['uuid']

        body_list = [{"image_info": {}}, {"image": {"id": "111"}},
                     {"image_invalid": {"inv": None}}, {"image_info": {"id": None}}]
        for each in body_list:
            url = "/ironic/v1/nodes/{0}/vendor_passthru/cache_image".format(node_id)
            response = self.successResultOf(request(
                self, self.root, "POST", url, body=json.dumps(each)))
            self.assertEqual(response.code, 400)

    def test_vendor_passthru_cache_image(self):
        """
        Test ``/nodes/<node-id>/vendor_passthru/cache_image`` returns a 202 and
        sets the cache_image_id and cache_status on the node
        """
        new_node = self.create_node({'properties': {'memory_mb': 131072}})
        node_id = new_node['uuid']

        self.assertFalse(new_node['driver_info'].get('cache_image_id'))
        self.assertFalse(new_node['driver_info'].get('cache_status'))

        image_id = str(uuid4())
        body = {"image_info": {"id": image_id}}
        url = self.url + "/{0}/vendor_passthru/cache_image".format(node_id)
        response = self.successResultOf(request(
            self, self.root, "POST", url, body=json.dumps(body)))
        self.assertEqual(response.code, 202)

        # GET node and verify the cache attributes on `driver_info`
        content = self.get_nodes('/' + str(node_id))
        self.assertEqual(content['driver_info']['cache_image_id'], image_id)
        self.assertEqual(content['driver_info']['cache_status'], 'cached')

    def test_vendor_passthru_cache_image_list_nodes(self):
        """
        Test ``/nodes/<node-id>/vendor_passthru/cache_image`` returns a 202 and
        sets the cache_image_id and cache_status on the node, and not the other nodes.
        """
        content1 = self.get_nodes('/detail')
        node_id = content1['nodes'].pop()['uuid']

        image_id = str(uuid4())
        body = {"image_info": {"id": image_id}}
        url = self.url + "/{0}/vendor_passthru/cache_image".format(node_id)
        response = self.successResultOf(request(
            self, self.root, "POST", url, body=json.dumps(body)))
        self.assertEqual(response.code, 202)

        # GET node and verify the cache attributes on `driver_info`
        content = self.get_nodes('/' + str(node_id))
        self.assertEqual(content['driver_info']['cache_image_id'], image_id)
        self.assertEqual(content['driver_info']['cache_status'], 'cached')

        # verify the other nodes are not cached
        content2 = self.get_nodes('/detail')
        self.assertTrue(content1, content2)
        non_cached_nodes = [each for each in content2['nodes']]
        non_cached_nodes.pop()
        for each in non_cached_nodes:
            self.assertFalse(each['driver_info']['cache_image_id'])
            self.assertFalse(each['driver_info']['cache_status'])



"""
API mock for Rackspace Queues.
"""
import json
import collections
from uuid import uuid4
from six import text_type

from mimic.imimic import IAPIMock
from twisted.plugin import IPlugin
from mimic.canned_responses.queue import(add_queue, list_queues,
                                         delete_queue)

from mimic.catalog import Entry
from mimic.catalog import Endpoint
from mimic.rest.mimicapp import MimicApp
from zope.interface import implementer
from random import randrange


@implementer(IAPIMock, IPlugin)
class QueueApi(object):
    """
    API mock for Queues.
    """
    def __init__(self, regions=["ORD", "DFW", "IAD"]):
        """
        Create a QueueApi with an empty region cache
        """
        self._regions = regions

    def resource_for_region(self, uri_prefix, region, session_store):
        """
        Get an :obj:`twisted.web.iweb.IResource` for the given URI prefix;
        implement :obj:`IAPIMock`.
        """
        return (QueueApiRoutes(self, uri_prefix, session_store, region).app.resource())

    def catalog_entries(self, tenant_id):
        """
        List catalog entries for the Nova API.
        """
        return [
            Entry(
                tenant_id, "rax:queues", "cloudQueues",
                [
                    Endpoint(tenant_id, region, text_type(uuid4()),
                             prefix="v1")
                    for region in self._regions
                ]
            )
        ]


class Q_Cache(dict):
    """
    Sketch: A replacement for queue_cache-as-dictionary,
    queue_cache-as-object-with-methods-and-attributes.  It's still a dictionary so
    that we can continue to treat it as one in the slightly crufty
    canned_responses module that expects dumb data structures rather than a
    structured object.
    """


class QueueApiRoutes(object):
    """
    Klein routes for queue API methods.
    """

    app = MimicApp()

    def __init__(self, api_mock, uri_prefix, session_store, queue_name):
        """
        Create a queue region with a given URI prefix (used for generating URIs
        to queues).
        """
        self.uri_prefix = uri_prefix
        self._api_mock = api_mock
        self._session_store = session_store
        self._queue_name = queue_name

    def _queue_cache(self, tenant_id):
        """
        Get the given queue-cache object for the given tenant, creating one if
        there isn't one.
        """
        return (self._session_store.session_for_tenant_id(tenant_id)
                .data_for_api(self._api_mock,
                              lambda: collections.defaultdict(Q_Cache))
                [self._queue_name])

    @app.route("/v1/<string:tenant_id>/queues/<string:queue_name>", methods=['PUT'])
    def create_queue(self, request, tenant_id, queue_name):
        """
        Api call to create and save queue. HTTP status code of 201.
        """
        queue_id = randrange(99999)
        q_cache = self._queue_cache(tenant_id)
        response_data = add_queue(
            queue_id, queue_name,
            tenant_id, q_cache)
        request.setResponseCode(response_data[1])
        return json.dumps(response_data[0])

    @app.route("/v1/<string:tenant_id>/queues", methods=['GET'])
    def list_queues(self, request, tenant_id):
        """
        Api call to get a list of queues. HTTP status code of 200
        """
        q_cache = self._queue_cache(tenant_id)
        response_data = list_queues(tenant_id, q_cache)
        request.setResponseCode(response_data[1])
        return json.dumps(response_data[0])

    @app.route("/v1/<string:tenant_id>/queues/<string:queue_name>", methods=['DELETE'])
    def del_queue(self, request, tenant_id, queue_name):
        """
        Api call to delete a queue. HTTP status code of 201
        """
        q_cache = self._queue_cache(tenant_id)
        response_data = delete_queue(queue_name, q_cache)
        request.setResponseCode(response_data[1])
        return json.dumps(response_data[0])



"""
Cannned responses for glance images
"""
from mimic.canned_responses.json.glance.glance_images_json import (images,
                                                                   image_schema)


def get_images():
    """
    Canned response for glance images list call
    """
    return images


def get_image_schema():
    """
    Canned response for GET glance image schema API call
    """
    return image_schema
